@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -19,45 +19,67 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "AddFileThread.h"
-#endif
-
 #include <wx/defs.h>		// Needed before any other wx/*.h
 #include <wx/event.h>		// Needed for wxCommandEvent
 #include <wx/timer.h>		// Needed for wxStopWatch
-
-#if wxCHECK_VERSION(2, 5, 2)
-#	include <wx/arrstr.h>		// Needed for wxArrayString
-#endif
-
+#include <wx/arrstr.h>		// Needed for wxArrayString
 #include <wx/filename.h>	// Needed for wxFileName::GetPathSeperator()
 #include <wx/utils.h>
 
 #include "AddFileThread.h"	// Interface declarations
-#include "StringFunctions.h"	// Needed for unicode2char
-#include "amule.h"			// Needed for theApp
+#include <common/StringFunctions.h>	// Needed for unicode2char
+#include "amule.h"		// Needed for theApp
 #include "OPCodes.h"		// Needed for TM_HASHTHREADFINISHED
 #include "PartFile.h"		// Needed for CKnownFile and CPartFile
-#include "CFile.h"			// Needed for CFile
+#include "CFile.h"		// Needed for CFile
+#include "FileFunctions.h" // Needed for GetLastModificationTime()
 #include "AICHSyncThread.h"	// Needed for CAICHSyncThread
-#include "Logger.h"			// Needed for AddLogLine
-#include "Format.h"
+#include "Logger.h"		// Needed for AddLogLine
+#include <common/Format.h>
+#include "InternalEvents.h"	// Needed for wxMuleInternalEvent
 
 #include <algorithm>
-
+#include <list>
 
 const byte default_zero_hash[16] = { 0x31, 0xD6, 0xCF, 0xE0, 0xD1, 0x6A, 0xE9, 0x31, 
 												0xB7, 0x3C, 0x59, 0xD7, 0xE0, 0xC0, 0x89, 0xC0 };
 
 
-std::list<CAddFileThread::QueuedFile> CAddFileThread::s_queue;
-bool CAddFileThread::s_running;
-CAddFileThread*	CAddFileThread::s_thread;
-wxMutex CAddFileThread::s_mutex;
+
+//! The mutex used to protect variables and datastructures used by the thread.
+static wxMutex		s_mutex;
+
+//! Is the hasher active. Does not mean that there are any threads running.
+static bool			s_running;
+
+//
+static CAddFileThread*		s_thread;
+
+
+/**
+ * Container for queued files.
+ */
+struct QueuedFile
+{
+	//! The full path to the file.
+	wxString			m_path;
+	//! The name of the file.
+	wxString			m_name;
+	//! The PartFile owning this file in case of a final hashing (completing).
+	const CPartFile*	m_owner;
+};
+
+
+//! The queue-type
+typedef std::list<QueuedFile> FileQueue;
+//! The queue of files to be hashed
+static FileQueue s_queue;
+
+
+
 
 CAddFileThread::CAddFileThread()
 	: wxThread(wxTHREAD_DETACHED)
@@ -83,25 +105,30 @@ void CAddFileThread::CreateNewThread()
 		AddLogLineM( false, _("Hasher: Creating new thread.") );
 		
 		s_thread = new CAddFileThread();
-	
-		switch ( s_thread->Create() ) {
-			case wxTHREAD_NO_ERROR:
-				break;
-			case wxTHREAD_RUNNING:
-				AddDebugLogLineM( true, logHasher, wxT("Error, attempt to create a already running thread!") );
-				break;
-			case wxTHREAD_NO_RESOURCE:
-				AddDebugLogLineM( true, logHasher, wxT("Error, attempt to create a thread without resources!") );
-				break;
-			default:
-				AddDebugLogLineM( true, logHasher, wxT("Error, unknown error attempting to create a thread!") );
-		}
 
+		wxThreadError state = s_thread->Create();
+		if (state != wxTHREAD_NO_ERROR) {
+			AddDebugLogLineM(true, logHasher, wxString::Format(wxT("Error while creating file-hashing thread: %d"), state));
+		
+			// Delete() must be called in this case acording to the docs.
+			s_thread->Delete();
+			delete s_thread;
+			s_thread = NULL;
+		}
+		
 		// The threads shouldn't be hugging the CPU, as it will already be hugging the HD
 		s_thread->SetPriority(WXTHREAD_MIN_PRIORITY);
 		
-		s_thread->Run();
-	}
+		state = s_thread->Run();
+		if (state != wxTHREAD_NO_ERROR) {
+			AddDebugLogLineM(true, logHasher, wxString::Format(wxT("Error while attempting to run file-hashing thread: %d"), state));
+			
+			// Delete() must be called in this case acording to the docs.
+			s_thread->Delete();
+			delete s_thread;
+			s_thread = NULL;
+		}		
+	}	
 }
 
 
@@ -126,37 +153,28 @@ void CAddFileThread::Stop()
 {
 	wxASSERT( wxThread::IsMain() );
 
-	s_mutex.Lock();
-
-	if ( s_running ) {
-		s_running = false;
-			
-		s_mutex.Unlock();
-		
-		if ( s_thread ) {
-			AddLogLineM( false, _("Hasher: Signaling for remaining threads to terminate.") );
-			
-			// We will be blocking the main thread, so we need to leave the
-			// gui mutex, so that events can still be processed while we are
-			// waiting.
-			wxMutexGuiLeave();
-			
-			// Wait for all threads to die
-			while ( s_thread ) {
-				// Sleep for 1/100 of a second to avoid clobbering the mutex
-				// By doing this we ensure that this function only returns
-				// once the thread has died.
-
-				otherfunctions::MilliSleep(10);
-			}
-
-			// Re-claim the GUI mutex.
-			wxMutexGuiLeave();
+	{
+		wxMutexLocker lock(s_mutex);
+	
+		if ( !s_running ) {
+			AddDebugLogLineM( true, logHasher, wxT("Warning, Attempted to stop already stopped hasher!") );
+			return;
 		}
-	} else {
-		AddDebugLogLineM( true, logHasher, wxT("Warning, Attempted to stop already stopped hasher!") );
+	
+		s_running = false;
+	}
+	
+	if ( s_thread ) {
+		AddLogLineM( false, _("Hasher: Signaling for remaining threads to terminate.") );
+		
+		// Wait for all threads to die
+		while ( s_thread ) {
+			// Sleep for 1/100 of a second to avoid clobbering the mutex
+			MilliSleep(10);
 
-		s_mutex.Unlock();
+			// Flush any pending log-entries caused by the thread.
+			CLogger::FlushPendingEntries();
+		}
 	}
 }
 
@@ -204,21 +222,20 @@ wxThread::ExitCode CAddFileThread::Entry()
 
 	// Continue to loop until there's nothing to do, or someone kills the threads
 	while ( IsRunning() ) {
-		s_mutex.Lock();
-		if ( s_queue.size() ) {
+		{
+			wxMutexLocker lock(s_mutex);
+
+			if ( s_queue.empty() ) {
+				AddLogLineM( false, _("Hasher: No files on queue, stopping thread.") );
+				break;
+			}
+
 			current = s_queue.front();
 			s_queue.pop_front();
-
-			s_mutex.Unlock();
-		} else {
-			s_mutex.Unlock();
-			
-			AddLogLineM( false, _("Hasher: No files on queue, stopping thread.") );
-			break;
 		}
 
 
-		wxString filename = current.m_path + wxFileName::GetPathSeparator() + current.m_name;
+		wxString filename = JoinPaths(current.m_path, current.m_name);
 		
 		// The file currently getting hashed
 		CFile file;
@@ -231,7 +248,7 @@ wxThread::ExitCode CAddFileThread::Entry()
 
 
 		// We only support files <= 4gigs
-		if ( (uint64)file.GetLength() >= (uint64)(4294967295U) ) {
+		if (file.GetLength() > MAX_FILE_SIZE) {
 			AddDebugLogLineM( true, logHasher, wxT("Warning, file is bigger than 4GB, skipping: ") + filename );
 			continue;
 		}
@@ -248,7 +265,7 @@ wxThread::ExitCode CAddFileThread::Entry()
 		
 		// Set initial values
 		knownfile->m_strFilePath = current.m_path;
-		knownfile->m_strFileName = current.m_name;
+		knownfile->SetFileName(current.m_name);
 		
 		knownfile->SetFileSize( file.GetLength() );
 		knownfile->date = GetLastModificationTime( filename );
@@ -299,16 +316,15 @@ wxThread::ExitCode CAddFileThread::Entry()
 				knownfile->hashlist.Clear();
 			} else {
 				unsigned int len = knownfile->hashlist.GetCount() * 16;
-				byte* data = new byte[ len ];
+				byte data[len];
 				
 				for (size_t i = 0; i < knownfile->hashlist.GetCount(); i++) {
-					memcpy( data + 16*i, knownfile->hashlist[i], 16 );
+					memcpy( data + 16*i, knownfile->hashlist[i].GetHash(), 16 );
 				}
 	
 				byte hash[16];
 	
 				knownfile->CreateHashFromString( data, len, hash, NULL );
-				delete [] data;
 
 				knownfile->m_abyFileHash.SetHash( (byte*)hash );
 			}
@@ -346,9 +362,10 @@ wxThread::ExitCode CAddFileThread::Entry()
 	AddLogLineM( false, _("Hasher: A thread has died.") );
 
 	
-	s_mutex.Lock();
-	s_thread = NULL;
-	s_mutex.Unlock();
+	{
+		wxMutexLocker lock(s_mutex);
+		s_thread = NULL;
+	}
 
 
 	// Notify the core.
@@ -382,8 +399,11 @@ bool CAddFileThread::CreateNextPartHash( CFile* file, CKnownFile* owner, bool cr
 	byte* data = new byte[cur_length];
 		
 	// Check for read errors
-	if ( file->Read(data, cur_length) != cur_length ) {
+	try {
+		file->Read(data, cur_length);
+	} catch (const CIOFailureException& e) {
 		delete[] data;
+		AddDebugLogLineM(true, logHasher, wxT("IO failure while hashing file: ") + e.what());
 		
 		return false;
 	}
@@ -405,9 +425,10 @@ bool CAddFileThread::CreateNextPartHash( CFile* file, CKnownFile* owner, bool cr
 
 	// Create the md4 hash and perhaps a AICH hash
 	owner->CreateHashFromString( data, cur_length, hash, pBlockAICHHashTree );
+	delete[] data;
 	
 	// Store the md4 hash
-	owner->hashlist.Add( (byte*)hash );
+	owner->hashlist.Add( CMD4Hash(hash) );
 
 	
 	// Kry This is because of the ed2k implementation for parts. A 2 * PARTSIZE 
@@ -415,11 +436,8 @@ bool CAddFileThread::CreateNextPartHash( CFile* file, CKnownFile* owner, bool cr
 	// So we have to create the hash for the 0-size data, which will be the default
 	// md4 hash for null data: 31D6CFE0D16AE931B73C59D7E0C089C0	
 	if ( zero_hash ) {
-		owner->hashlist.Add( default_zero_hash );
+		owner->hashlist.Add( CMD4Hash(default_zero_hash) );
 	}
 
-	delete[] data;
-
 	return true;
 }
-
