@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -24,17 +24,9 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-//
-// FileDetailDialog.cpp : implementation file
-//
-
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "FileDetailDialog.h"
-#endif
-
 #include <wx/settings.h>	// Needed for wxSYS_COLOUR_WINDOW
 #include <wx/stattext.h>	// Needed for wxStaticText
 #include <wx/sizer.h>
@@ -49,7 +41,9 @@
 #include "updownclient.h"	// Needed for CUpDownClient
 #include "PartFile.h"		// Needed for CPartFile
 #include "Color.h"		// Needed for SYSCOLOR
-#include "Format.h"		// Needed for CFormat
+#include <common/Format.h>		// Needed for CFormat
+#include "amule.h"				// Needed for theApp
+#include "SharedFileList.h"		// Needed for CSharedFileList
 
 #define ID_MY_TIMER 1652
 
@@ -64,7 +58,6 @@ BEGIN_EVENT_TABLE(CFileDetailDialog,wxDialog)
 	EVT_TIMER(ID_MY_TIMER,CFileDetailDialog::OnTimer)
 END_EVENT_TABLE()
 
-using namespace otherfunctions;
 
 CFileDetailDialog::CFileDetailDialog(wxWindow* parent,CPartFile* file)
 : wxDialog(parent,-1,_("File Details"),wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER )
@@ -76,6 +69,9 @@ CFileDetailDialog::CFileDetailDialog(wxWindow* parent,CPartFile* file)
 	UpdateData();
 	content->SetSizeHints(this);
 	content->Show(this,TRUE);
+
+	// Currently renaming of completed files causes problem with kad
+	FindWindow(IDC_RENAME)->Enable(file->IsPartFile());
 }
 
 CFileDetailDialog::~CFileDetailDialog()
@@ -103,7 +99,7 @@ void CFileDetailDialog::UpdateData()
 		CastChild(IDC_FILENAME,wxTextCtrl)->SetValue(m_file->GetFileName());
 	}
 
-	CastChild(IDC_FHASH,wxStaticText)->SetLabel(EncodeBase16(m_file->GetFileHash(), 16));
+	CastChild(IDC_FHASH,wxStaticText)->SetLabel(m_file->GetFileHash().Encode());
 	CastChild(IDC_FSIZE,wxControl)->SetLabel(CastItoXBytes(m_file->GetFileSize()));
 	CastChild(IDC_PFSTATUS,wxControl)->SetLabel(m_file->getPartfileStatus());
 	bufferS = wxString::Format(wxT("%i (%i)"),m_file->GetPartCount(),m_file->GetHashCount());
@@ -208,22 +204,24 @@ void CFileDetailDialog::FillSourcenameList()
 
 void CFileDetailDialog::OnBnClickedShowComment(wxCommandEvent& WXUNUSED(evt))
 {
-	CCommentDialogLst* dialog=new CCommentDialogLst(this,m_file);
-	dialog->ShowModal();
-	delete dialog;
+	CCommentDialogLst(this,m_file).ShowModal();
 }
 
+
 void CFileDetailDialog::OnBnClickedRename(wxCommandEvent& WXUNUSED(evt))
 {
-	wxString NewFileName=CastChild(IDC_FILENAME,wxTextCtrl)->GetValue();
-	m_file->SetFileName(NewFileName);
-	m_file->SavePartFile(); 
-	FindWindow(IDC_FNAME)->SetLabel(MakeStringEscaped(m_file->GetFileName()));
-	FindWindow(IDC_METFILE)->SetLabel(m_file->GetFullName());
-	
-	CastChild( IDC_FILENAME, wxTextCtrl )->SetValue(m_file->GetFileName());
+	wxString fileName = CastChild(IDC_FILENAME, wxTextCtrl)->GetValue();
 
-	Layout();
+	if (!fileName.IsEmpty() and (fileName != m_file->GetFileName())) {
+		if (theApp.sharedfiles->RenameFile(m_file, fileName)) {
+			FindWindow(IDC_FNAME)->SetLabel(MakeStringEscaped(m_file->GetFileName()));
+			FindWindow(IDC_METFILE)->SetLabel(m_file->GetFullName());
+	
+			CastChild(IDC_FILENAME, wxTextCtrl)->SetValue(m_file->GetFileName());
+	
+			Layout();
+		}
+	}
 }
 
 
