@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Tiku
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,37 +20,33 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-//
-// HTTPDownload.cpp : implementation file
-//
-
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "HTTPDownload.h"
-#endif
-
 #ifdef __WXMSW__
 	#include <wx/defs.h>
 	#include <wx/msw/winundef.h>
 #endif
 
+
 #include <wx/intl.h>
-#include <cmath>
+#include <wx/wfstream.h>
 #include <wx/protocol/http.h>
+#include <cmath>
+#include <memory>
 
 #include "amule.h"
 #include "HTTPDownload.h"	// Interface declarations
-#ifndef AMULE_DAEMON
-#include "inetdownload.h"	// Needed for inetDownload
-#endif
-#include "StringFunctions.h"	// Needed for unicode2char
+#include <common/StringFunctions.h>	// Needed for unicode2char
 #include "OtherFunctions.h" 	// Needed for CastChild
-#include "Logger.h"				// Needed for AddLogLine*
-#include "Format.h"				// Needed for CFormat
+#include "Logger.h"		// Needed for AddLogLine*
+#include <common/Format.h>		// Needed for CFormat
+#include "InternalEvents.h"	// Needed for wxMuleInternalEvent
+#include "Proxy.h"
+#include "Preferences.h"
 
 #ifndef AMULE_DAEMON 
+	#include "inetdownload.h"	// Needed for inetDownload
 	#include "muuli_wdr.h"		// Needed for ID_CANCEL: Let it here or will fail on win32
 	#include "MuleGifCtrl.h"
 	#include <wx/sizer.h> 
@@ -58,13 +54,15 @@
 	#include <wx/stattext.h>
 #endif
 
+
+
 #ifndef AMULE_DAEMON 
 BEGIN_EVENT_TABLE(CHTTPDownloadThreadDlg,wxDialog)
-  EVT_BUTTON(ID_CANCEL,CHTTPDownloadThreadDlg::OnBtnCancel)
+  EVT_BUTTON(ID_HTTPCANCEL, CHTTPDownloadThreadDlg::OnBtnCancel)
 END_EVENT_TABLE()
 
 CHTTPDownloadThreadDlg::CHTTPDownloadThreadDlg(wxWindow* parent, CHTTPDownloadThread* thread)
-  : wxDialog(parent,1025,_("Downloading..."),wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE|wxSYSTEM_MENU)
+  : wxDialog(parent, -1,_("Downloading..."),wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE|wxSYSTEM_MENU)
 {
 	downloadDlg(this,TRUE)->Show(this,TRUE);
 	
@@ -82,6 +80,7 @@ CHTTPDownloadThreadDlg::CHTTPDownloadThreadDlg(wxWindow* parent, CHTTPDownloadTh
 
 void CHTTPDownloadThreadDlg::OnBtnCancel(wxCommandEvent& WXUNUSED(evt))
 {
+	printf("HTTP download cancelled\n");
  	m_parent_thread->Delete();
 }
 
@@ -102,9 +101,9 @@ CHTTPDownloadThreadDlg::~CHTTPDownloadThreadDlg()
 
 void CHTTPDownloadThreadDlg::UpdateGauge(int dltotal,int dlnow) 
 {	
-	wxString label = wxT("( ") + otherfunctions::CastItoXBytes(dlnow) + wxT(" / ");
+	wxString label = wxT("( ") + CastItoXBytes(dlnow) + wxT(" / ");
 	if (dltotal > 0) {
-		label += otherfunctions::CastItoXBytes(dltotal);
+		label += CastItoXBytes(dltotal);
 	} else {
 		label += _("Unknown");
 	}
@@ -126,31 +125,25 @@ void CHTTPDownloadThreadDlg::UpdateGauge(int dltotal,int dlnow)
 CHTTPDownloadThreadGUI::CHTTPDownloadThreadGUI(wxString urlname, wxString filename,HTTP_Download_File file_id)
 : CHTTPDownloadThreadBase(urlname,filename,file_id)
 {
-	#if !defined(__WXMAC__)
-		m_myDlg= new CHTTPDownloadThreadDlg(theApp.GetTopWindow(), this);
-		m_myDlg->Show(true);
-	#endif
+	m_myDlg= new CHTTPDownloadThreadDlg(theApp.GetTopWindow(), this);
+	m_myDlg->Show(true);
 }
 
 CHTTPDownloadThreadGUI::~CHTTPDownloadThreadGUI()
 {	
-	#if !defined(__WXMAC__)
-		wxMutexGuiEnter();
-		if (m_myDlg != NULL) {
-			m_myDlg->StopAnimation();
-			delete m_myDlg;
-		}
-		wxMutexGuiLeave();
-	#endif	
+	wxMutexGuiEnter();
+	if (m_myDlg != NULL) {
+		m_myDlg->StopAnimation();
+		m_myDlg->Destroy();
+	}
+	wxMutexGuiLeave();
 }
 
 void CHTTPDownloadThreadGUI::ProgressCallback(int dltotal, int dlnow) 
 {
-	#if !defined(__WXMAC__)
-		wxMutexGuiEnter();
-		m_myDlg->UpdateGauge(dltotal,dlnow);
-		wxMutexGuiLeave();
-	#endif	
+	wxMutexGuiEnter();
+	m_myDlg->UpdateGauge(dltotal,dlnow);
+	wxMutexGuiLeave();
 }
 
 #endif
@@ -176,13 +169,19 @@ wxThread::ExitCode CHTTPDownloadThreadBase::Entry()
 		return NULL;
 	}
 	
-	FILE *outfile = NULL; 
+	wxHTTP* url_handler = NULL;
+	wxInputStream* url_read_stream = NULL;
+	
+	printf("HTTP download thread started\n");
+	
+	const CProxyData* proxy_data = thePrefs::GetProxyData();
+	bool use_proxy = proxy_data != NULL && proxy_data->m_proxyEnable;
 	
 	try {	
 		
-		outfile = fopen(unicode2char(m_tempfile), "w");
+		wxFFileOutputStream outfile(m_tempfile);
 		
-		if (outfile == NULL) {
+		if (!outfile.Ok()) {
 			throw(wxString(CFormat(wxT("Unable to create destination file %s for download!\n")) % m_tempfile));
 		}
 			
@@ -190,15 +189,15 @@ wxThread::ExitCode CHTTPDownloadThreadBase::Entry()
 			// Nowhere to download from!
 			throw(wxString(wxT("The URL to download can't be empty\n")));
 		}
+
+		url_handler = new wxHTTP;
+		
+		url_handler->SetProxyMode(use_proxy);
 	
-		wxHTTP url_handler;
-		wxInputStream* url_read_stream = GetInputStream(url_handler, m_url);
-		if (!url_read_stream) {					
-			#if wxCHECK_VERSION(2,5,1)			
-				throw(wxString(CFormat(wxT("The URL %s returned: %i - Error (%i)!")) % m_url % url_handler.GetResponse() % url_handler.GetError()));
-			#else
-				throw(wxString(CFormat(wxT("The URL %s returned a error!")) % m_url));
-			#endif
+		url_read_stream = GetInputStream(&url_handler, m_url,use_proxy);
+		
+		if (!url_read_stream) {
+			throw(wxString(CFormat(wxT("The URL %s returned: %i - Error (%i)!")) % m_url % url_handler->GetResponse() % url_handler->GetError()));
 		}
 		
 		int download_size = url_read_stream->GetSize();
@@ -216,7 +215,8 @@ wxThread::ExitCode CHTTPDownloadThreadBase::Entry()
 			current_read = url_read_stream->LastRead();
 			if (current_read) {
 				total_read += current_read;
-				int current_write = fwrite(buffer,1,current_read,outfile);
+				outfile.Write(buffer,current_read);
+				int current_write = outfile.LastWrite();
 				if (current_read != current_write) {
 					throw(wxString(wxT("Critical error while writing downloaded file")));
 				} else {
@@ -224,20 +224,22 @@ wxThread::ExitCode CHTTPDownloadThreadBase::Entry()
 				}
 			}
 		} while (current_read && !TestDestroy());
-		
-		delete url_read_stream;
-		
-		fclose(outfile);
-		
-	} catch (wxString& download_error) {
-		if (outfile) {
-			fclose(outfile);
+	} catch (const wxString& download_error) {
+		if (wxFileExists(m_tempfile)) {
 			wxRemoveFile(m_tempfile);
 		}
 		m_result = -1;		
 		AddLogLineM(false,download_error);
 	}
 
+	if (url_read_stream) {
+		delete url_read_stream;
+	}
+
+	if (url_handler) {
+		url_handler->Destroy();
+	}
+	
 	printf("HTTP download thread end\n");
 	
 	return 0;
@@ -249,10 +251,10 @@ void CHTTPDownloadThreadBase::OnExit()
 	wxMuleInternalEvent evt(wxEVT_CORE_FINISHED_HTTP_DOWNLOAD);
 	evt.SetInt((int)m_file_type);
 	evt.SetExtraLong((long)m_result);
-	wxPostEvent(&theApp,evt);		
+	wxPostEvent(&theApp,evt);
 }
 
-wxInputStream* CHTTPDownloadThreadBase::GetInputStream(wxHTTP& url_handler, const wxString& location) {
+wxInputStream* CHTTPDownloadThreadBase::GetInputStream(wxHTTP** url_handler, const wxString& location, bool proxy) {
 	// This function's purpose is to handle redirections in a proper way.
 	
 	if (TestDestroy()) {
@@ -300,60 +302,49 @@ wxInputStream* CHTTPDownloadThreadBase::GetInputStream(wxHTTP& url_handler, cons
 		host = host.BeforeFirst(wxT(':'));
 	}
 
-	#if wxCHECK_VERSION(2,5,1)
-		url_handler.Connect(host, port);
-	#else
-		wxIPV4address addr;
-		addr.Hostname(host);
-		addr.Service(port);
-		url_handler.Connect(addr, true);
-	#endif
+	wxIPV4address addr;
+	addr.Hostname(host);
+	addr.Service(port);
+	if (!(*url_handler)->Connect(addr, true)) {
+		throw wxString(wxT("Unable to connect to http download server"));		
+	}
+
+	wxInputStream* url_read_stream = (*url_handler)->GetInputStream(url);
 
-	wxInputStream* url_read_stream = url_handler.GetInputStream(url);
 	printf("Host: %s:%i\n",(const char*)unicode2char(host),port);
 	printf("URL: %s\n",(const char*)unicode2char(url));
+	printf("Response: %i (Error: %i)\n",(*url_handler)->GetResponse(), (*url_handler)->GetError());
+
+	if (!(*url_handler)->GetResponse()) {
+		printf("WARNING: Void response on stream creation\n");
+		// WTF? Why does this happen?
+		// This is probably produced by an already existing connection, because
+		// the input stream is created nevertheless. However, data is not the same.
+		delete url_read_stream;
+		throw wxString(wxT("Invalid response from http download server"));
+	}
+
+	if ((*url_handler)->GetResponse() == 301  // Moved permanently
+		|| (*url_handler)->GetResponse() == 302 // Moved temporarily
+		// What about 300 (multiple choices)? Do we have to handle it?
+		) { 
 		
-	#if wxCHECK_VERSION(2,5,1) 
-		printf("Response: %i (Error: %i)\n",url_handler.GetResponse(), url_handler.GetError());
-	
-		if (!url_handler.GetResponse()) {
-			printf("WARNING: Void response on stream creation\n");
-			// WTF? Why does this happen?
-			// I've seen it crash here, sadly.
-			// This is probably produced by an already existing connection, because
-			// the input stream is created nevertheless. However, data is not the same.
-			delete url_read_stream;
-			throw wxString(wxT("Invalid response from http download server"));
-		}
-	
-		if (url_handler.GetResponse() == 301  // Moved permanently
-			|| url_handler.GetResponse() == 302 // Moved temporarily
-			// What about 300 (multiple choices)? Do we have to handle it?
-			) { 
+		// We have to remove the current stream.
+		delete url_read_stream;
 			
-			// We have to remove the current stream.
-			delete url_read_stream;
-				
-			wxString new_location = url_handler.GetHeader(wxT("Location"));
-			if (!new_location.IsEmpty()) {
-				printf("Redirected\n");
-				url_read_stream = GetInputStream(url_handler, new_location);
-			} else {
-				printf("ERROR: Redirection code received with no URL\n");
-				url_read_stream = NULL;
-			}
-		}
+		wxString new_location = (*url_handler)->GetHeader(wxT("Location"));
 		
-	#else
-		// This is NOT safe at all. Just a workaround - wx2.4 does not have wxHTTP::GetResponse
-		wxString new_location = url_handler.GetHeader(wxT("Location"));
+		(*url_handler)->Destroy();
 		if (!new_location.IsEmpty()) {
-			delete url_read_stream;
-			// Maybe a 301/302
-			printf("Redirected?\n");
-			url_read_stream = GetInputStream(url_handler, new_location);
+			(*url_handler) = new wxHTTP;
+			(*url_handler)->SetProxyMode(proxy);
+			url_read_stream = GetInputStream(url_handler, new_location, proxy);
+		} else {
+			printf("ERROR: Redirection code received with no URL\n");
+			url_handler = NULL;
+			url_read_stream = NULL;
 		}
-	#endif
-	
+	}
+		
 	return url_read_stream;
 }
