@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,45 +20,38 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #include <cstdio>
 #include <cstdlib>
 
 #include <wx/defs.h>
-
-#if wxCHECK_VERSION(2,5,1)
-	#include <wx/stopwatch.h>
-#endif
-
+#include <wx/stopwatch.h>
 #include <wx/timer.h> // Needed for wxGetLocalTimeMillis
-
 #include <wx/tokenzr.h>
 #include <wx/filename.h>
 #include <wx/textfile.h>
 #include <wx/config.h>
+#include <wx/utils.h>
 
 #ifdef __WXMSW__
 	#include <wx/msw/winundef.h>
 #endif
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "Preferences.h"
-#endif
-
 #include "amule.h"
 #ifdef HAVE_CONFIG_H
-#include "config.h"		// Needed for PACKAGE_STRING
+	#include "config.h"		// Needed for PACKAGE_STRING
 #endif
 #include "OtherFunctions.h"
-#include "StringFunctions.h"
+#include <common/StringFunctions.h>
 #include "OPCodes.h"		// Needed for PREFFILE_VERSION
 #include "Preferences.h"
 #include "CFile.h"
-#include "MD5Sum.h"
+#include <common/MD5Sum.h>
 #include "Logger.h"
-#include "Format.h"		// Needed for CFormat
+#include <common/Format.h>		// Needed for CFormat
+#include <common/PlatformSpecific.h>	// Needed for GetDocumentsDir()
 
 #ifndef AMULE_DAEMON
 #include <wx/valgen.h>
@@ -67,9 +60,9 @@
 #include "muuli_wdr.h"
 #include "StatisticsDlg.h"
 #include <wx/choice.h>
+#include <algorithm>		// Needed for std::sort
 #endif
 
-using namespace otherfunctions;
 
 #define DEFAULT_TCP_PORT 4662
 
@@ -89,6 +82,7 @@ wxString	CPreferences::s_nick;
 uint16		CPreferences::s_maxupload;
 uint16		CPreferences::s_maxdownload;
 uint16		CPreferences::s_slotallocation;
+wxString	CPreferences::s_Addr;
 uint16		CPreferences::s_port;
 uint16		CPreferences::s_udpport;
 bool		CPreferences::s_UDPDisable;
@@ -120,7 +114,6 @@ uint16		CPreferences::s_OSUpdate;
 uint64		CPreferences::s_totalDownloadedBytes;
 uint64		CPreferences::s_totalUploadedBytes;
 wxString	CPreferences::s_languageID;
-bool		CPreferences::s_transferDoubleclick;
 uint8		CPreferences::s_iSeeShares;
 uint8		CPreferences::s_iToolDelayTime;
 uint8		CPreferences::s_splitterbarPosition;
@@ -166,16 +159,16 @@ bool		CPreferences::s_bWebEnabled;
 bool		CPreferences::s_bWebUseGzip;
 uint32		CPreferences::s_nWebPageRefresh;
 bool		CPreferences::s_bWebLowEnabled;
+wxString	CPreferences::s_WebTemplate;
 bool		CPreferences::s_showCatTabInfos;
 uint32		CPreferences::s_allcatType;
-uint32		CPreferences::s_desktopMode;
 uint8		CPreferences::s_NoNeededSources;
 bool		CPreferences::s_DropFullQueueSources;
 bool		CPreferences::s_DropHighQueueRankingSources;
 uint32		CPreferences::s_HighQueueRanking;
 uint32		CPreferences::s_AutoDropTimer;
 bool 		CPreferences::s_AcceptExternalConnections;
-bool 		CPreferences::s_ECUseTCPPort;
+wxString	CPreferences::s_ECAddr;
 uint32		CPreferences::s_ECPort;
 wxString	CPreferences::s_ECPassword;
 bool		CPreferences::s_IPFilterOn;
@@ -193,6 +186,7 @@ wxString	CPreferences::s_OSDirectory;
 wxString	CPreferences::s_SkinFile;
 bool		CPreferences::s_UseSkinFile;
 bool		CPreferences::s_FastED2KLinksHandler;
+bool		CPreferences::s_ToolbarOrientation;
 int		CPreferences::s_perms_files;
 int		CPreferences::s_perms_dirs;
 bool		CPreferences::s_AICHTrustEveryHash;
@@ -206,6 +200,11 @@ bool		CPreferences::s_FilterSomeMessages;
 bool		CPreferences::s_ShareHiddenFiles;
 bool		CPreferences::s_AutoSortDownload;
 bool		CPreferences::s_NewVersionCheck;
+bool		CPreferences::s_ConnectToKad;
+bool		CPreferences::s_ConnectToED2K;
+unsigned	CPreferences::s_maxClientVersions;
+bool		CPreferences::s_ExecOnCompletion;
+wxString	CPreferences::s_ExecOnCompletionCommand;
 
 /**
  * Template Cfg class for connecting with widgets.
@@ -640,7 +639,7 @@ public:
 			int id = aMuleLanguages[m_selection].id;
 		
 			// save language selection
-			thePrefs::SetLanguageID(otherfunctions::wxLang2Str(id));
+			thePrefs::SetLanguageID(wxLang2Str(id));
 
 			return true;
 		}
@@ -661,7 +660,7 @@ public:
 		langSelector->Clear();
 
 		m_selection = 0;
-		int wxId = otherfunctions::StrLang2wx(thePrefs::GetLanguageID());
+		int wxId = StrLang2wx(thePrefs::GetLanguageID());
 	
 		// Add all other languages in alphabetical order
 		// and find the index of the selected language.
@@ -696,23 +695,13 @@ CPreferences::CPreferences()
 	CFile preffile;
 	if ( wxFileExists( fullpath ) ) {
 		if ( preffile.Open(fullpath, CFile::read) ) {
-			Preferences_Ext_Struct prefsExt;
-	
-			memset( &prefsExt, 0, sizeof(Preferences_Ext_Struct) );
-			// NOTE: This Read is dangerous.  It doesn't do The Right Thing
-			// with respect to endianness for the fields of prefsExt.  At this
-			// time, it doesn't matter because no fields except the version and
-			// userhash are used and they aren't sensitive to endianness.  If
-			// other fields are used in the future, this will have to be revisited.
-			off_t read = preffile.Read( &prefsExt, sizeof(Preferences_Ext_Struct) );
-
-			if ( read == sizeof(Preferences_Ext_Struct) ) {
-				md4cpy(s_userhash, prefsExt.userhash);
-			} else {
+			try {
+				preffile.ReadUInt8(); // Version. Value is not used.
+				s_userhash = preffile.ReadHash();
+			} catch (const CSafeIOException& e) {
+				AddDebugLogLineM(true, logGeneral, wxT("Error while reading userhash: ") + e.what());
 				SetStandartValues();
 			}
-			
-			preffile.Close();	
 		} else {
 			SetStandartValues();
 		}
@@ -787,9 +776,9 @@ void CPreferences::BuildItemList( const wxString& appdir )
 		wxString	customBrowser; // left empty
 	#endif
 
-	NewCfgItem(IDC_FCHECK,		(MkCfg_Int( wxT("/FakeCheck/Browser"), s_Browser, browser )));
-	NewCfgItem(IDC_FCHECKTABS,	(new Cfg_Bool( wxT("/FakeCheck/BrowserTab"), s_BrowserTab, true )));
-	NewCfgItem(IDC_FCHECKSELF,	(new Cfg_Str(  wxT("/FakeCheck/CustomBrowser"), s_CustomBrowser, customBrowser )));
+	NewCfgItem(IDC_BROWSER,		(MkCfg_Int( wxT("/FakeCheck/Browser"), s_Browser, browser )));
+	NewCfgItem(IDC_BROWSERTABS,	(new Cfg_Bool( wxT("/FakeCheck/BrowserTab"), s_BrowserTab, true )));
+	NewCfgItem(IDC_BROWSERSELF,	(new Cfg_Str(  wxT("/FakeCheck/CustomBrowser"), s_CustomBrowser, customBrowser )));
 	NewCfgItem(IDC_QUEUESIZE,	(MkCfg_Int( wxT("/eMule/QueueSizePref"), s_iQueueSize, 50 )));
 
 
@@ -841,16 +830,24 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	NewCfgItem(IDC_SAFESERVERCONNECT,	(new Cfg_Bool( wxT("/eMule/SafeServerConnect"), s_safeServerConnect, false )));
 	NewCfgItem(IDC_AUTOCONNECTSTATICONLY,	(new Cfg_Bool( wxT("/eMule/AutoConnectStaticOnly"), s_autoconnectstaticonly, false )));
 	NewCfgItem(IDC_SMARTIDCHECK,	(new Cfg_Bool( wxT("/eMule/SmartIdCheck"), s_smartidcheck, true )));
+	// Enabled networks
+	NewCfgItem( IDC_NETWORKKAD, (new Cfg_Bool( wxT("/eMule/ConnectToKad"),	s_ConnectToKad, true )) );
+	NewCfgItem( IDC_NETWORKED2K, ( new Cfg_Bool( wxT("/eMule/ConnectToED2K"),	s_ConnectToED2K, true ) ));
+
 
 	/**
 	 * Files
 	 **/
 	NewCfgItem(IDC_TEMPFILES,	(new Cfg_Str(  wxT("/eMule/TempDir"), 	s_tempdir, appdir + wxT("Temp") )));
 	
-	#ifdef __WXMAC__
-		wxString incpath = wxGetHomeDir() + wxFileName::GetPathSeparator() + wxT("Documents") + wxFileName::GetPathSeparator() + wxT("aMule Downloads");
-	#elif defined(__WXMSW__)
-		wxString incpath = wxGetHomeDir() + wxFileName::GetPathSeparator() + wxT("aMule Downloads");
+	#if defined(__WXMAC__) || defined(__WINDOWS__)
+		wxString incpath = GetDocumentsDir();
+		if (incpath.IsEmpty()) {
+			// There is a built-in possibility for this call to fail, though I can't imagine a reason for that.
+			incpath = appdir + wxT("Incoming");
+		} else {
+			incpath = JoinPaths(incpath, wxT("aMule Downloads"));
+		}
 	#else 
 		wxString incpath = appdir + wxT("Incoming");
 	#endif
@@ -858,9 +855,6 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	
 	NewCfgItem(IDC_ICH,		(new Cfg_Bool( wxT("/eMule/ICH"), s_ICH, true )));
 	NewCfgItem(IDC_AICHTRUST,	(new Cfg_Bool( wxT("/eMule/AICHTrust"), s_AICHTrustEveryHash, false )));
-	NewCfgItem(IDC_METADATA,	(new Cfg_Bool( wxT("/ExternalConnect/ExtractMetaDataTags"), s_ExtractMetaData, false )));
-	NewCfgItem(IDC_CHUNKALLOC,	(new Cfg_Bool( wxT("/ExternalConnect/FullChunkAlloc"), s_AllocFullChunk, false )));
-	NewCfgItem(IDC_FULLALLOCATE,	(new Cfg_Bool( wxT("/ExternalConnect/FullPartAlloc"), s_AllocFullPart, false )));
 	NewCfgItem(IDC_CHECKDISKSPACE,	(new Cfg_Bool( wxT("/eMule/CheckDiskspace"), s_checkDiskspace, true )));
 	NewCfgItem(IDC_MINDISKSPACE,	(MkCfg_Int( wxT("/eMule/MinFreeDiskSpace"), s_uMinFreeDiskSpace, 1 )));
 	NewCfgItem(IDC_ADDNEWFILESPAUSED,	(new Cfg_Bool( wxT("/eMule/AddNewFilesPaused"), s_addnewfilespaused, false )));
@@ -886,8 +880,9 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	NewCfgItem(IDC_WEB_GZIP,	(new Cfg_Bool( wxT("/WebServer/UseGzip"), s_bWebUseGzip, true )));
 	NewCfgItem(IDC_ENABLE_WEB_LOW,	(new Cfg_Bool( wxT("/WebServer/UseLowRightsUser"), s_bWebLowEnabled, false )));
 	NewCfgItem(IDC_WEB_REFRESH_TIMEOUT,	(MkCfg_Int( wxT("/WebServer/PageRefreshTime"), s_nWebPageRefresh, 120 )));
+	NewCfgItem(IDC_WEBTEMPLATE,	(new Cfg_Str( wxT("/WebServer/Template"), s_WebTemplate, wxEmptyString )));
 	NewCfgItem(IDC_EXT_CONN_ACCEPT,	(new Cfg_Bool( wxT("/ExternalConnect/AcceptExternalConnections"), s_AcceptExternalConnections, false )));
-	NewCfgItem(IDC_EXT_CONN_USETCP,	(new Cfg_Bool( wxT("/ExternalConnect/ECUseTCPPort"), s_ECUseTCPPort, false )));
+	NewCfgItem(IDC_EXT_CONN_IP,	(new Cfg_Str( wxT("/ExternalConnect/ECAddress"), s_ECAddr, wxEmptyString )));
 	NewCfgItem(IDC_EXT_CONN_TCP_PORT,	(MkCfg_Int( wxT("/ExternalConnect/ECPort"), s_ECPort, 4712 )));
 	NewCfgItem(IDC_EXT_CONN_PASSWD,	(new Cfg_Str_Encrypted( wxT("/ExternalConnect/ECPassword"), s_ECPassword, wxEmptyString )));
 
@@ -897,7 +892,6 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	NewCfgItem(IDC_ENABLETRAYICON,	(new Cfg_Bool( wxT("/eMule/EnableTrayIcon"), s_trayiconenabled, false )));
 	NewCfgItem(IDC_MINTRAY,		(new Cfg_Bool( wxT("/eMule/MinToTray"), s_mintotray, false )));
 	NewCfgItem(IDC_EXIT,		(new Cfg_Bool( wxT("/eMule/ConfirmExit"), s_confirmExit, false )));
-	NewCfgItem(IDC_DBLCLICK,	(new Cfg_Bool( wxT("/eMule/TransferDoubleClick"), s_transferDoubleclick, true )));
 	NewCfgItem(IDC_STARTMIN,	(new Cfg_Bool( wxT("/eMule/StartupMinimized"), s_startMinimized, false )));
 
 	/**
@@ -913,6 +907,7 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	NewCfgItem(IDC_USESKIN,		(new Cfg_Bool( wxT("/SkinGUIOptions/UseSkinFile"), s_UseSkinFile, false )));
 	NewCfgItem(IDC_SKINFILE,	(new Cfg_Str(  wxT("/SkinGUIOptions/SkinFile"), s_SkinFile, wxEmptyString )));
 	NewCfgItem(IDC_SHOWRATEONTITLE,	(new Cfg_Bool( wxT("/eMule/ShowRatesOnTitle"), s_ShowRatesOnTitle, false )));
+	NewCfgItem(IDC_VERTTOOLBAR,	(new Cfg_Bool( wxT("/eMule/VerticalToolbar"), s_ToolbarOrientation, false )));
 	
 	/**
 	 * External Apps
@@ -929,6 +924,7 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	NewCfgItem(IDC_UPLOAD_CAP,	(MkCfg_Int( wxT("/eMule/UploadCapacity"), s_maxGraphUploadRate, 3 )));
 	NewCfgItem(IDC_SLIDER3,		(MkCfg_Int( wxT("/eMule/StatsAverageMinutes"), s_statsAverageMinutes, 5 )));
 	NewCfgItem(IDC_SLIDER4,		(MkCfg_Int( wxT("/eMule/VariousStatisticsMaxValue"), s_statsMax, 100 )));
+	NewCfgItem(IDC_CLIENTVERSIONS,	(MkCfg_Int( wxT("/Statistics/MaxClientVersions"), s_maxClientVersions, 0 )));
 
 	/**
 	 * Sources
@@ -976,6 +972,12 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	 **/
 	 NewCfgItem(IDC_NEWVERSION,	(new Cfg_Bool( wxT("/eMule/NewVersionCheck"), s_NewVersionCheck, false )));
 	 
+	/**
+	 * Execute command on file completion.
+	 **/
+	NewCfgItem(ID_ONCMPLT,			(new Cfg_Bool( wxT("/eMule/ExecOnCompletion"), s_ExecOnCompletion, false)));
+	NewCfgItem(ID_ONCMPLT_TEXT,		(new Cfg_Str(  wxT("/eMule/ExecOnCompletionCommand"), s_ExecOnCompletionCommand, wxT("sh -c '~/example.sh \"%FILE\" %HASH'"))));
+	 
 	/**
 	 * The following doesn't have an associated widget.
 	 **/
@@ -993,9 +995,9 @@ void CPreferences::BuildItemList( const wxString& appdir )
 	s_MiscList.push_back( new Cfg_Bool( wxT("/eMule/DisableQueueList"),		s_bDisableQueueList, false ) );
 	s_MiscList.push_back(    MkCfg_Int( wxT("/eMule/MaxMessageSessions"),		s_maxmsgsessions, 50 ) );
 
-	s_MiscList.push_back(	 MkCfg_Int( wxT("/Statistics/DesktopMode"), s_desktopMode, 4 ) );
 	s_MiscList.push_back(	 MkCfg_Int( wxT("/eMule/PermissionsFiles"),	s_perms_files, 0640 ) );
 	s_MiscList.push_back(	 MkCfg_Int( wxT("/eMule/PermissionsDirs"),	s_perms_dirs, 0750 ) );
+	s_MiscList.push_back( new Cfg_Str(  wxT("/eMule/Address"),			s_Addr,	wxEmptyString ) );
 
 #ifndef AMULE_DAEMON
 	// Colors have been moved from global prefs to CStatisticsDlg
@@ -1178,36 +1180,23 @@ void CPreferences::SetStandartValues()
 	Save();
 }
 
-bool CPreferences::Save()
+
+void CPreferences::Save()
 {
 	wxString fullpath(theApp.ConfigDir + wxT("preferences.dat"));
 
-	bool error = false;
-
 	CFile preffile;
+	if (!wxFileExists(fullpath)) {
+		preffile.Create(fullpath);
+	}
 	
-	if ( !wxFileExists( fullpath ) )
-		preffile.Create( fullpath );
-	
-	if ( preffile.Open(fullpath, CFile::read_write) ) {
-		Preferences_Ext_Struct prefsExt;
-		memset( &prefsExt, 0, sizeof(Preferences_Ext_Struct) );
-		
-		prefsExt.version = PREFFILE_VERSION;
-		md4cpy( prefsExt.userhash, s_userhash.GetHash() );
-		
-		// NOTE: This Write is dangerous.  It doesn't do The Right Thing
-		// with respect to endianness for the fields of prefsExt.  At this
-		// time, it doesn't matter because no fields except the version and
-		// userhash are used and they aren't sensitive to endianness.  If
-		// other fields are used in the future, this will have to be revisited.
-		off_t written = preffile.Write( &prefsExt, sizeof(Preferences_Ext_Struct) );
-
-		error = written != sizeof(Preferences_Ext_Struct);
-		
-		preffile.Close();
-	} else {
-		error = true;
+	if (preffile.Open(fullpath, CFile::read_write)) {
+		try {
+			preffile.WriteUInt8(PREFFILE_VERSION);
+			preffile.WriteHash(s_userhash);
+		} catch (const CIOFailureException& e) {
+			AddDebugLogLineM(true, logGeneral, wxT("IO failure while saving user-hash: ") + e.what());
+		}
 	}
 
 	SavePreferences();
@@ -1215,20 +1204,14 @@ bool CPreferences::Save()
 	wxString shareddir(theApp.ConfigDir + wxT("shareddir.dat"));
 
 	wxRemoveFile(shareddir);
-
 	wxTextFile sdirfile(shareddir);
-
-	if(sdirfile.Create()) {
+	if (sdirfile.Create()) {
 		for(unsigned int ii = 0; ii < shareddir_list.GetCount(); ++ii) {
 			sdirfile.AddLine(shareddir_list[ii]);
 		}
-		sdirfile.Write(),
-		sdirfile.Close();
-	} else {
-		error = true;
+		
+		sdirfile.Write();
 	}
-
-	return error;
 }
 
 
@@ -1318,13 +1301,14 @@ void CPreferences::LoadPreferences()
 }
 
 
-void CPreferences::LoadCats() {
+void CPreferences::LoadCats()
+{
 	// default cat ... Meow! =(^.^)=
-	Category_Struct* newcat = new Category_Struct;
-	newcat->prio = 0;
-	newcat->color = 0;
+	Category_Struct* defaultcat = new Category_Struct;
+	defaultcat->prio = 0;
+	defaultcat->color = 0;
 
-	AddCat( newcat );
+	AddCat( defaultcat );
 
 	wxConfigBase* cfg = wxConfigBase::Get();
 
@@ -1418,6 +1402,34 @@ uint32 CPreferences::CPreferences::GetCatColor(size_t index)
 	return m_CatList[index]->color;
 }
 
+Category_Struct *CPreferences::CreateCategory(wxString name, wxString path, wxString comment, uint32 color, uint8 prio)
+{
+	Category_Struct *category = new Category_Struct();
+	category->incomingpath	= path;
+	category->title			= name;
+	category->comment		= comment;
+	category->color			= color;
+	category->prio			= prio;
+			
+	AddCat(category);
+	
+	SaveCats();
+	
+	return category;
+}
+
+void CPreferences::UpdateCategory(uint8 cat, wxString name, wxString path, wxString comment, uint32 color, uint8 prio)
+{
+	Category_Struct *category = m_CatList[cat];
+
+	category->incomingpath	= path;
+	category->title			= name;
+	category->comment		= comment;
+	category->color			= color;
+	category->prio			= prio;
+	
+	SaveCats();
+}
 
 // Jacobo221 - Several issues on the browsers:
 // netscape is named Netscape on some systems
@@ -1516,3 +1528,27 @@ void CPreferences::ReloadSharedFolders() {
 		sdirfile.Close();
 	}
 }
+
+bool CPreferences::IsMessageFiltered(const wxString& message) { 
+	if (s_FilterAllMessages) { 
+		return true;
+	} else {
+		if (s_FilterSomeMessages) {
+			if (s_MessageFilterString.IsSameAs(wxT("*"))){  
+				// Filter anything
+				return true;
+			} else {
+				wxStringTokenizer tokenizer( s_MessageFilterString, wxT(",") );
+				while (tokenizer.HasMoreTokens()) {
+					if ( message.Lower().Trim(false).Trim(true).Contains(
+							tokenizer.GetNextToken().Lower().Trim(false).Trim(true))) {
+						return true;
+					}
+				}
+				return false;
+			}			
+		} else {
+			return false;
+		}
+	}
+}
