@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,13 +20,9 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "SearchListCtrl.h"
-#endif
-
 #include "SearchListCtrl.h"	// Interface declarations
 #include "MuleNotebook.h"	// Needed for CMuleNotebook
 #include "DownloadQueue.h"	// Needed for CDownloadQueue
@@ -36,33 +32,47 @@
 #include "SearchList.h"		// Needed for CSearchFile
 #include "SearchDlg.h"		// Needed for CSearchDlg
 #include "amuleDlg.h"		// Needed for CamuleDlg
-#include "OPCodes.h"		// Needed for MP_RESUME
+#include "OPCodes.h"		// Needed for MP_ASSIGNCAT
 #include "amule.h"			// Needed for theApp
+#include "Color.h"			// Needed for BLEND and SYSCOLOR
+#include "muuli_wdr.h"		// Needed for clientImages
+#include "Preferences.h"	// Needed for thePrefs
 
 #include <wx/menu.h>
 #include <wx/settings.h>
 #include <algorithm>
 
 
+// For arrow-pixmaps
+#include "pixmaps/sort_dn.xpm"
+#include "pixmaps/sort_up.xpm"
+#include "pixmaps/sort_dnx2.xpm"
+#include "pixmaps/sort_upx2.xpm"
+
+
+
 BEGIN_EVENT_TABLE(CSearchListCtrl, CMuleListCtrl)
-	EVT_RIGHT_DOWN(CSearchListCtrl::OnRightClick)
+	EVT_LIST_ITEM_RIGHT_CLICK(-1, CSearchListCtrl::OnRightClick)
 	EVT_LIST_COL_CLICK( -1,       CSearchListCtrl::OnColumnLClick)
 	EVT_LIST_COL_END_DRAG( -1,    CSearchListCtrl::OnColumnResize)
 
 	EVT_MENU( MP_GETED2KLINK,     CSearchListCtrl::OnPopupGetUrl)
-	EVT_MENU( MP_GETHTMLED2KLINK, CSearchListCtrl::OnPopupGetUrl)
-	EVT_MENU( MP_FAKECHECK1,      CSearchListCtrl::OnPopupFakeCheck)
-	EVT_MENU( MP_FAKECHECK2,      CSearchListCtrl::OnPopupFakeCheck)
 	EVT_MENU( MP_RAZORSTATS,      CSearchListCtrl::OnRazorStatsCheck)
 	EVT_MENU( MP_RESUME,          CSearchListCtrl::OnPopupDownload)
+	EVT_MENU_RANGE( MP_ASSIGNCAT, MP_ASSIGNCAT + 99, CSearchListCtrl::OnPopupDownload )
 
 	EVT_LIST_ITEM_ACTIVATED( -1,  CSearchListCtrl::OnItemActivated)
 END_EVENT_TABLE()
 
-using namespace otherfunctions;
+//! Shared list of arrow-pixmaps + 
+static wxImageListType s_imgList(16, 16, true, 0);
+//! The index of the first rating icon
+static int s_ratingOffset = 0;
+
 
 std::list<CSearchListCtrl*> CSearchListCtrl::s_lists;
 
+
 enum SearchListColumns {
 	ID_SEARCH_COL_NAME = 0,
 	ID_SEARCH_COL_SIZE,
@@ -71,8 +81,12 @@ enum SearchListColumns {
 	ID_SEARCH_COL_FILEID
 };
 
+
 CSearchListCtrl::CSearchListCtrl( wxWindow* parent, wxWindowID winid, const wxPoint& pos, const wxSize& size, long style, const wxValidator& validator, const wxString& name )
-	: CMuleListCtrl( parent, winid, pos, size, style, validator, name )
+	: CMuleListCtrl( parent, winid, pos, size, style, validator, name),
+	  m_filterKnown(false),
+	  m_invert(false),
+	  m_filterEnabled(false)
 {
 	// Setting the sorter function.
 	SetSortFunc( SortProc );
@@ -101,6 +115,23 @@ CSearchListCtrl::CSearchListCtrl( wxWindow* parent, wxWindowID winid, const wxPo
 
 	// Add the list so that it will be synced with the other lists
 	s_lists.push_back( this );
+
+	
+	if (s_imgList.GetImageCount() == 0) {
+		s_imgList.Add(wxBitmap(sort_dn_xpm));
+		s_imgList.Add(wxBitmap(sort_up_xpm));
+		s_imgList.Add(wxBitmap(sort_dnx2_xpm));
+		s_imgList.Add(wxBitmap(sort_upx2_xpm));
+		
+		// Add rating icons
+		s_ratingOffset = s_imgList.Add(wxBitmap(clientImages(Client_InvalidRating_Smiley)));
+		s_imgList.Add(wxBitmap(clientImages(Client_PoorRating_Smiley)));
+		s_imgList.Add(wxBitmap(clientImages(Client_GoodRating_Smiley)));
+		s_imgList.Add(wxBitmap(clientImages(Client_FairRating_Smiley)));
+		s_imgList.Add(wxBitmap(clientImages(Client_ExcellentRating_Smiley)));		
+	}
+
+	SetImageList(&s_imgList, wxIMAGE_LIST_SMALL);
 }
 
 
@@ -125,8 +156,21 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	if ( toshow->GetSearchID() != m_nResultsID )
 		return;
 
+	// Check if the result should be shown
+	if (!IsFiltered(toshow)) {
+		m_filteredOut.push_back(toshow);
+		
+		return;
+	}
+
+	// Rating
+	int image = -1;
+	if (toshow->HasRating()) {
+		image = s_ratingOffset + toshow->UserRating() - 1;
+	}
+	
 	// Insert the item before the item found by the search
-	uint32 newid = InsertItem( GetInsertPos( (long)toshow ), toshow->GetFileName() );
+	uint32 newid = InsertItem( GetInsertPos( (long)toshow ), toshow->GetFileName(), image );
 
 	SetItemData( newid, (long)toshow );
 
@@ -144,25 +188,29 @@ void CSearchListCtrl::AddResult(CSearchFile* toshow)
 	SetItem(newid, ID_SEARCH_COL_FILEID, toshow->GetFileHash().Encode() );
 
 	// Set the color of the item
-	UpdateColor( newid );
+	UpdateItemColor( newid );
 }
 
 
 void CSearchListCtrl::UpdateResult(CSearchFile* toupdate)
 {
-	long index = FindItem( -1, (long)toupdate );
-
-	if ( index != -1 ) {
-		// Remove the old item
-		DeleteItem( index );
-
-		// Re-add it to update the displayed information and have it properly positioned
-		AddResult( toupdate );
+	long index = FindItem(-1, (long)toupdate);
+	if (index != -1) {
+		wxString temp = wxString::Format( wxT("%d (%d)"), toupdate->GetSourceCount(), toupdate->GetCompleteSourceCount());
+		SetItem(index, ID_SEARCH_COL_SOURCES, temp);
+		
+		UpdateItemColor(index);
+		
+		// Deletions of items causes rather large ammount of flicker, so to
+		// avoid this, we resort the list to ensure correct ordering.
+		if (!IsItemSorted(index)) {
+			SortList();
+		}
 	}
 }
 
 
-void CSearchListCtrl::UpdateColor( long index )
+void CSearchListCtrl::UpdateItemColor( long index )
 {
 	wxListItem item;
 	item.SetId( index );
@@ -204,7 +252,7 @@ void CSearchListCtrl::UpdateColor( long index )
 		newitem.SetId( index );
 		newitem.SetTextColour( wxColour( red, green, blue ) );
 		newitem.SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_LISTBOX));
-		SetItem( newitem );
+		SetItem( newitem );	
 	}
 }
 
@@ -230,40 +278,110 @@ void CSearchListCtrl::ShowResults( long ResultsID )
 
 long CSearchListCtrl::GetSearchId()
 {
-		return m_nResultsID;
+	return m_nResultsID;
 }
 
 
-int CSearchListCtrl::SortProc( long item1, long item2, long sortData )
+void CSearchListCtrl::SetFilter(const wxString& regExp, bool invert, bool filterKnown)
+{
+	if (regExp.IsEmpty()) {
+		// Show everything
+		m_filterText = wxT(".*");
+	} else {
+		m_filterText = regExp;
+	}
+	
+	m_filter.Compile(m_filterText, wxRE_DEFAULT | wxRE_ICASE);
+	m_filterKnown = filterKnown;
+	m_invert = invert;
+	
+	if (m_filterEnabled) {
+		// Swap the list of filtered results so we can freely add new items to the list
+		ResultList curFiltered;
+		std::swap(curFiltered, m_filteredOut);
+	
+		// Filter items already on the list
+		for (int i = 0; i < GetItemCount();) {
+			CSearchFile* file = (CSearchFile*)GetItemData(i);
+		
+			if (IsFiltered(file)) {
+				++i;
+			} else {
+				m_filteredOut.push_back(file);
+				DeleteItem(i);
+			}
+		}
+
+		// Check the previously filtered items.
+		ResultList::iterator it = curFiltered.begin();
+		for (; it != curFiltered.end(); ++it) {
+			if (IsFiltered(*it)) {
+				AddResult(*it);
+			} else {
+				m_filteredOut.push_back(*it);
+			}
+		}
+	}
+}
+
+
+void CSearchListCtrl::EnableFiltering(bool enabled)
+{
+	if (enabled != m_filterEnabled) {
+		m_filterEnabled = enabled;
+		
+		if (enabled) {
+			SetFilter(m_filterText, m_invert, m_filterKnown);
+		} else {
+			ResultList::iterator it = m_filteredOut.begin();
+			for (; it != m_filteredOut.end(); ++it) {
+				AddResult(*it);
+			}
+
+			m_filteredOut.clear();
+		}
+	}
+}
+
+
+size_t CSearchListCtrl::GetHiddenItemCount() const
+{
+	return m_filteredOut.size();
+}
+
+
+bool CSearchListCtrl::IsFiltered(const CSearchFile* file)
+{
+	// By default, everything is displayed
+	bool result = true;
+	
+	if (m_filterEnabled && m_filter.IsValid()) {
+		result = m_filter.Matches(file->GetFileName());
+		result = ((result && !m_invert) || (!result && m_invert));
+	
+		if (result && m_filterKnown) {
+			result = !theApp.downloadqueue->GetFileByID(file->GetFileHash());
+
+			if (result) {
+				result = !theApp.knownfiles->FindKnownFileByID(file->GetFileHash());
+			}
+		}
+	}
+
+	return result;
+}
+
+
+int CSearchListCtrl::SortProc(long item1, long item2, long sortData)
 {
 	CSearchFile* file1 = (CSearchFile*)item1;
 	CSearchFile* file2 = (CSearchFile*)item2;
 
 	// Modifies the result, 1 for ascending, -1 for decending
-	int modifier = 1;
-	bool alternate = false;
+	int modifier = (sortData & CMuleListCtrl::SORT_DES) ? -1 : 1;
+	bool alternate = (sortData & CMuleListCtrl::SORT_ALT);
 
-	wxASSERT( SORT_OFFSET_ASC < SORT_OFFSET_DEC );
-	wxASSERT( SORT_OFFSET_DEC < SORT_OFFSET_ALT_ASC );
-	wxASSERT( SORT_OFFSET_ALT_ASC < SORT_OFFSET_ALT_DEC );
-
-	if ( sortData >= SORT_OFFSET_ALT_DEC ) {
-		modifier = -1;
-		alternate = true;
-		sortData -= SORT_OFFSET_ALT_DEC;
-	}
-
-	if ( sortData >= SORT_OFFSET_ALT_ASC ) {
-		alternate = true;
-		sortData -= SORT_OFFSET_ALT_ASC;
-	}
-
-	if ( sortData >= SORT_OFFSET_DEC ) {
-		modifier = -1;
-		sortData -= SORT_OFFSET_DEC;
-	}
-
-	switch ( sortData ) {
+	switch (sortData & CMuleListCtrl::COLUMN_MASK) {
 		// Sort by filename
 		case ID_SEARCH_COL_NAME:
 			return modifier * file1->GetFileName().CmpNoCase( file2->GetFileName() );
@@ -294,8 +412,18 @@ int CSearchListCtrl::SortProc( long item1, long item2, long sortData )
 		}
 		
 		// Sort by file-types
-		case ID_SEARCH_COL_TYPE:
-			return modifier *  GetFiletypeByName( file1->GetFileName() ).Cmp(GetFiletypeByName(file2->GetFileName()));
+		case ID_SEARCH_COL_TYPE: {
+			wxString fileName1 = wxT(".") + file1->GetFileName().AfterLast('.').MakeLower();
+			wxString fileName2 = wxT(".") + file2->GetFileName().AfterLast('.').MakeLower();
+			
+			int result = GetFiletypeByName(fileName1).Cmp(GetFiletypeByName(fileName2));
+			if (result == 0) {
+				// Same file-type, sort by extension
+				result = fileName1.Cmp(fileName2);
+			}
+
+			return modifier * result;
+		}
 
 		// Sort by file-hash
 		case ID_SEARCH_COL_FILEID:
@@ -321,13 +449,10 @@ void CSearchListCtrl::SyncLists( CSearchListCtrl* src, CSearchListCtrl* dst )
 	}
 
 	// Sync sorting
-	if ( src->GetSortColumn() != dst->GetSortColumn() ||
-	     src->GetSortAsc() != dst->GetSortAsc() )
-	{
-		dst->SetSortColumn( src->GetSortColumn() );
-		dst->SetSortAsc( src->GetSortAsc() );
-		dst->SetSortAlt( src->GetSortAlt() );
-
+	unsigned column = src->GetSortColumn();
+	unsigned order  = src->GetSortOrder();
+	if (column != dst->GetSortColumn() || order != dst->GetSortOrder()) {
+		dst->SetSorting(column, order);
 		dst->SortList();
 	}
 }
@@ -345,36 +470,34 @@ void CSearchListCtrl::SyncOtherLists( CSearchListCtrl* src )
 }
 
 
-void CSearchListCtrl::OnRightClick(wxMouseEvent& event)
+void CSearchListCtrl::OnRightClick(wxListEvent& event)
 {
-
 	CheckSelection(event);
 	
-	if ( GetSelectedItemCount() != 0 ) {
-
+	if (GetSelectedItemCount()) {
 		// Create the popup-menu
-		wxMenu* menu = new wxMenu( _("File") );
-		menu->Append( MP_RESUME, _("Download"));
-		menu->Append( MP_GETED2KLINK, _("Copy ED2k link to clipboard"));
-		menu->Append( MP_GETHTMLED2KLINK, _("Copy ED2k link to clipboard (HTML)"));
-		menu->AppendSeparator();
-		menu->Append( MP_RAZORSTATS, _("Get Razorback 2's stats for this file"));
-		menu->AppendSeparator();
-		menu->Append( MP_FAKECHECK2, _("jugle.net Fake Check")); // deltahf -> fakecheck
-		menu->Append( MP_FAKECHECK1, _("'Donkey Fakes' Fake Check"));
-		menu->AppendSeparator();
+		wxMenu menu(_("File"));
+		menu.Append(MP_RESUME, _("Download"));
+		
+		wxMenu* cats = new wxMenu(_("Category"));
+		cats->Append(MP_ASSIGNCAT, _("Main"));
+		for (unsigned i = 1; i < theApp.glob_prefs->GetCatCount(); i++) {
+			cats->Append(MP_ASSIGNCAT + i,
+				theApp.glob_prefs->GetCategory(i)->title);
+		}
+		
+		menu.Append(MP_MENU_CATS, _("Download in category"), cats);
+		menu.AppendSeparator();
+		menu.Append(MP_RAZORSTATS, _("Get Razorback 2's stats for this file"));
+		menu.AppendSeparator();
+		menu.Append(MP_GETED2KLINK, _("Copy ED2k link to clipboard"));
 
 		// These should only be enabled for single-selections
 		bool enable = GetSelectedItemCount();
-		menu->Enable( MP_GETED2KLINK, enable );
-		menu->Enable( MP_GETHTMLED2KLINK, enable );
-		menu->Enable( MP_FAKECHECK1, enable );
-		menu->Enable( MP_FAKECHECK2, enable );
-	
+		menu.Enable(MP_GETED2KLINK, enable);
+		menu.Enable(MP_MENU_CATS, (theApp.glob_prefs->GetCatCount() > 1));
 
-		PopupMenu( menu, event.GetPosition() );
-
-		delete menu;
+		PopupMenu(&menu, event.GetPoint());
 	} else {
 		event.Skip();
 	}
@@ -396,7 +519,7 @@ void CSearchListCtrl::OnColumnResize( wxListEvent& WXUNUSED(event) )
 }
 
 
-void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& event )
+void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& WXUNUSED(event) )
 {
 	wxString URIs;	
 	
@@ -405,45 +528,16 @@ void CSearchListCtrl::OnPopupGetUrl( wxCommandEvent& event )
 	while( index != -1 ) {
 		CSearchFile* file = (CSearchFile*)GetItemData( index );
 
-		switch ( event.GetId() ) {
-			case MP_GETED2KLINK:				URIs += theApp.CreateED2kLink( file ) + wxT("\n");					break;
-			case MP_GETHTMLED2KLINK:			URIs += theApp.CreateHTMLED2kLink( file ) + wxT("\n");				break;
-		}
+		URIs += theApp.CreateED2kLink( file ) + wxT("\n");
 
 		index = GetNextItem( index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
 	}
 
-	if ( !URIs.IsEmpty() ) {	
+	if ( !URIs.IsEmpty() ) {
 		theApp.CopyTextToClipboard( URIs.RemoveLast() );
 	}
-	
 }
 
-void CSearchListCtrl::OnPopupFakeCheck( wxCommandEvent& event )
-{
-	int item = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
-	if ( item == -1 ) {
-		return;
-	}
-
-	CSearchFile* file = (CSearchFile*)GetItemData( item );
-	wxString URL;
-
-	// Add new cases as nescesarry
-	switch ( event.GetId() ) {
-		case MP_FAKECHECK1:
-			URL = theApp.GenFakeCheckUrl( file );
-			break;
-		
-		case MP_FAKECHECK2:
-			URL = theApp.GenFakeCheckUrl2( file );
-			break;
-	}
-
-	if ( !URL.IsEmpty() ) {
-		theApp.amuledlg->LaunchUrl( URL );
-	}
-}
 
 void CSearchListCtrl::OnRazorStatsCheck( wxCommandEvent& WXUNUSED(event) )
 {
@@ -457,20 +551,25 @@ void CSearchListCtrl::OnRazorStatsCheck( wxCommandEvent& WXUNUSED(event) )
 }
 
 
-void CSearchListCtrl::OnPopupDownload( wxCommandEvent& WXUNUSED(event) )
+void CSearchListCtrl::OnPopupDownload(wxCommandEvent& event)
 {
-	wxCommandEvent nullEvt;
-	theApp.amuledlg->searchwnd->OnBnClickedDownload(nullEvt);
+	if (event.GetId() == MP_RESUME) {
+		// Via the "Download" menu-item, use category specified in drop-down menu
+		DownloadSelected();
+	} else {
+		// Via an "Download in category" item
+		DownloadSelected(event.GetId() - MP_ASSIGNCAT);
+	}
 }
 
 
 void CSearchListCtrl::OnItemActivated( wxListEvent& WXUNUSED(event) )
 {
-	wxCommandEvent nullEvt;
-	theApp.amuledlg->searchwnd->OnBnClickedDownload(nullEvt);
+	DownloadSelected();
 }
 
-bool CSearchListCtrl::AltSortAllowed( int column )
+
+bool CSearchListCtrl::AltSortAllowed(unsigned column) const
 {
 	switch ( column ) {
 		case ID_SEARCH_COL_SOURCES:
@@ -480,3 +579,30 @@ bool CSearchListCtrl::AltSortAllowed( int column )
 			return false;
 	}
 }
+
+
+void CSearchListCtrl::DownloadSelected(int category)
+{
+	FindWindowById(IDC_SDOWNLOAD)->Enable(FALSE);
+
+	// Either the "Download" menu-item, the download-button, double-click or enter
+	if (category == -1) {
+		// Defaults to main category
+		category = 0;
+		
+		if (CastByID(IDC_EXTENDEDSEARCHCHECK, NULL, wxCheckBox)->GetValue()) {
+			category = CastByID(ID_AUTOCATASSIGN, NULL, wxChoice)->GetSelection();
+		}		
+	}
+	
+	long index = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	while (index > -1) {
+		CoreNotify_Search_Add_Download((CSearchFile*)GetItemData(index), category);
+		
+		UpdateItemColor(index);
+
+		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+	}
+
+}
+
