@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,13 +20,9 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "SharedFilesCtrl.h"
-#endif
-
 #include "muuli_wdr.h"			// Needed for ID_SHFILELIST
 #include "SharedFilesCtrl.h"	// Interface declarations
 #include "OtherFunctions.h"		// Needed for CastItoXBytes
@@ -49,7 +45,7 @@
 
 
 BEGIN_EVENT_TABLE(CSharedFilesCtrl,CMuleListCtrl)
-	EVT_RIGHT_DOWN(CSharedFilesCtrl::OnRightClick)
+	EVT_LIST_ITEM_RIGHT_CLICK(-1, CSharedFilesCtrl::OnRightClick)
 
 	EVT_MENU( MP_PRIOVERYLOW,	CSharedFilesCtrl::OnSetPriority )
 	EVT_MENU( MP_PRIOLOW,		CSharedFilesCtrl::OnSetPriority )
@@ -60,13 +56,15 @@ BEGIN_EVENT_TABLE(CSharedFilesCtrl,CMuleListCtrl)
 	EVT_MENU( MP_PRIOAUTO,		CSharedFilesCtrl::OnSetPriorityAuto )
 
 	EVT_MENU( MP_CMT,			CSharedFilesCtrl::OnEditComment )
-	EVT_MENU( MP_GETCOMMENTS,   CSharedFilesCtrl::OnGetComment )  
 	EVT_MENU( MP_RAZORSTATS, 		CSharedFilesCtrl::OnGetRazorStats )	
 	EVT_MENU( MP_GETED2KLINK,				CSharedFilesCtrl::OnCreateURI )
-	EVT_MENU( MP_GETHTMLED2KLINK,			CSharedFilesCtrl::OnCreateURI )
 	EVT_MENU( MP_GETSOURCEED2KLINK,			CSharedFilesCtrl::OnCreateURI )
-	EVT_MENU( MP_GETAICHED2KLINK,				CSharedFilesCtrl::OnCreateURI )
 	EVT_MENU( MP_GETHOSTNAMESOURCEED2KLINK,	CSharedFilesCtrl::OnCreateURI )
+	EVT_MENU( MP_GETAICHED2KLINK,	CSharedFilesCtrl::OnCreateURI )
+	EVT_MENU( MP_RENAME,		CSharedFilesCtrl::OnRename )
+
+
+	EVT_CHAR( CSharedFilesCtrl::OnKeyPressed )
 END_EVENT_TABLE()
 
 enum SharedFilesListColumns {
@@ -83,7 +81,7 @@ enum SharedFilesListColumns {
 	ID_SHARED_COL_PATH
 };
 
-using namespace otherfunctions;
+
 CSharedFilesCtrl::CSharedFilesCtrl(wxWindow* parent, int id, const wxPoint& pos, wxSize size, int flags)
 	: CMuleListCtrl(parent, id, pos, size, flags | wxLC_OWNERDRAW )
 {
@@ -117,13 +115,12 @@ CSharedFilesCtrl::~CSharedFilesCtrl()
 }
 
 
-void CSharedFilesCtrl::OnRightClick(wxMouseEvent& event)
+void CSharedFilesCtrl::OnRightClick(wxListEvent& event)
 {
-
 	long item_hit = CheckSelection(event);
 
 	if ( (m_menu == NULL) && (item_hit != -1)) {
-		wxMenu* m_menu = new wxMenu(_("Shared Files"));
+		m_menu = new wxMenu(_("Shared Files"));
 		wxMenu* prioMenu = new wxMenu();
 		prioMenu->Append(MP_PRIOVERYLOW, _("Very low"));
 		prioMenu->Append(MP_PRIOLOW, _("Low"));
@@ -135,18 +132,28 @@ void CSharedFilesCtrl::OnRightClick(wxMouseEvent& event)
 
 		m_menu->Append(0,_("Priority"),prioMenu);
 		m_menu->AppendSeparator();
-		m_menu->Append(MP_CMT, _("Change this file's comment..."));
-		m_menu->Append(MP_GETCOMMENTS,_("Show all comments"));
+
+		CKnownFile* file = (CKnownFile*)GetItemData(item_hit);
+		if (file->GetFileComment().IsEmpty() && !file->GetFileRating()) {
+			m_menu->Append(MP_CMT, _("Add Comment/Rating"));
+		} else {
+			m_menu->Append(MP_CMT, _("Edit Comment/Rating"));
+		}
+		
+		m_menu->AppendSeparator();
+		m_menu->Append(MP_RENAME, _("Rename"));
 		m_menu->AppendSeparator();
 		m_menu->Append( MP_RAZORSTATS, _("Get Razorback 2's stats for this file"));
 		m_menu->AppendSeparator();
 		m_menu->Append(MP_GETED2KLINK,_("Copy ED2k &link to clipboard"));
 		m_menu->Append(MP_GETSOURCEED2KLINK,_("Copy ED2k link to clipboard (&Source)"));
 		m_menu->Append(MP_GETHOSTNAMESOURCEED2KLINK,_("Copy ED2k link to clipboard (Hostname)"));
-		m_menu->Append(MP_GETHTMLED2KLINK,_("Copy ED2k link to clipboard (&HTML)"));
-		m_menu->Append(MP_GETAICHED2KLINK,_("Copy ED2k link to clipboard (&AICH info)"));		
+		m_menu->Append(MP_GETAICHED2KLINK,_("Copy ED2k link to clipboard (&AICH info)"));
 
-		PopupMenu( m_menu, event.GetPosition() );
+		m_menu->Enable(MP_GETHOSTNAMESOURCEED2KLINK, !thePrefs::GetYourHostname().IsEmpty());
+		m_menu->Enable(MP_RENAME, file->IsPartFile());
+		
+		PopupMenu( m_menu, event.GetPoint() );
 
 		delete m_menu;
 
@@ -155,15 +162,18 @@ void CSharedFilesCtrl::OnRightClick(wxMouseEvent& event)
 }
 
 
+#ifndef CLIENT_GUI
 void CSharedFilesCtrl::ShowFileList(CSharedFileList* list)
 {
 	DeleteAllItems();
 
-	CKnownFileMap::iterator it = list->m_Files_map.begin();
-	for ( ; it != list->m_Files_map.end(); ++it ) {
-		ShowFile( it->second );
+	std::vector<CKnownFile*> files;
+	list->CopyFileList(files);
+	for (unsigned i = 0; i < files.size(); ++i) {
+		ShowFile( files[i] );
 	}
 }
+#endif
 
 
 void CSharedFilesCtrl::RemoveFile(CKnownFile *toRemove)
@@ -183,7 +193,8 @@ void CSharedFilesCtrl::UpdateFile( CKnownFile* file, long itemnr )
 	wxString buffer;
 	
 	SetItemData( itemnr, (long)file );
-	
+
+	SetItem(itemnr, ID_SHARED_COL_NAME, file->GetFileName() );
 	SetItem(itemnr, ID_SHARED_COL_TYPE, GetFiletypeByName(file->GetFileName()) );
 	SetItem(itemnr, ID_SHARED_COL_SIZE, CastItoXBytes(file->GetFileSize()) );
 	SetItem(itemnr, ID_SHARED_COL_PRIO, PriorityToStr( file->GetUpPriority(), file->IsAutoUpPriority() ) );
@@ -284,8 +295,8 @@ void CSharedFilesCtrl::OnCreateURI( wxCommandEvent& event )
 	wxString URIs;	
 
 	if ( event.GetId() == MP_GETSOURCEED2KLINK ) {
-		if ( !theApp.serverconnect->IsConnected() || theApp.serverconnect->IsLowID() ) {
-			wxMessageBox(_("You need a HighID to create a valid sourcelink"));
+		if ( !theApp.IsConnectedED2K() || theApp.serverconnect->IsLowID() ) {
+			wxMessageBox(_("You need a HighID to create a valid sourcelink"), _("Warning"), wxOK | wxICON_ERROR, this);
 
 			return;
 		}
@@ -298,10 +309,9 @@ void CSharedFilesCtrl::OnCreateURI( wxCommandEvent& event )
 
 		switch ( event.GetId() ) {
 			case MP_GETED2KLINK:				URIs += theApp.CreateED2kLink( file ) + wxT("\n");					break;
-			case MP_GETHTMLED2KLINK:			URIs += theApp.CreateHTMLED2kLink( file ) + wxT("\n");				break;
 			case MP_GETSOURCEED2KLINK:			URIs += theApp.CreateED2kSourceLink( file ) + wxT("\n");			break;
 			case MP_GETHOSTNAMESOURCEED2KLINK:	URIs += theApp.CreateED2kHostnameSourceLink( file ) + wxT("\n");	break;
-			case MP_GETAICHED2KLINK:	URIs += theApp.CreateED2kAICHLink( file ) + wxT("\n"); break;
+			case MP_GETAICHED2KLINK: URIs += theApp.CreateED2kAICHLink( file ) + wxT("\n");	break;
 		}
 
 		index = GetNextItem( index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
@@ -326,43 +336,29 @@ void CSharedFilesCtrl::OnEditComment( wxCommandEvent& WXUNUSED(event) )
 	}
 }
 
-void CSharedFilesCtrl::OnGetComment ( wxCommandEvent& WXUNUSED(event) )
-{
-	int item = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
-	if ( item != -1 ) {
-		CKnownFile* file = (CKnownFile*)GetItemData( item );
-	
-		theApp.amuledlg->LaunchUrl(wxT("http://jugle.net/?comments=") + file->GetFileHash().Encode());
-	}
-}
-	
-	
-int CSharedFilesCtrl::SortProc( long item1, long item2, long sortData )
+
+int CSharedFilesCtrl::SortProc(long item1, long item2, long sortData)
 {
 	CKnownFile* file1 = (CKnownFile*)item1;
 	CKnownFile* file2 = (CKnownFile*)item2;
 
-	int mod = 1;
-	if ( IsOffsetDec( sortData ) ) {
-		// Sorting is decending
-		mod = -1;
+	int mod = (sortData & CMuleListCtrl::SORT_DES) ? -1 : 1;
+	bool altSorting = (sortData & CMuleListCtrl::SORT_ALT);
 
-		// We only handle the ascending cases
-		sortData -= 1000;
-	}
-
-
-	switch ( sortData ) {
-		// Sort by filename. Ascending.
-		case  ID_SHARED_COL_NAME: return mod * file1->GetFileName().CmpNoCase( file2->GetFileName() );
+	switch (sortData & CMuleListCtrl::COLUMN_MASK) {
+		// Sort by filename.
+		case  ID_SHARED_COL_NAME:
+			return mod * file1->GetFileName().CmpNoCase( file2->GetFileName() );
 		
-		// Sort by filesize. Ascending.
-		case  ID_SHARED_COL_SIZE: return mod * CmpAny( file1->GetFileSize(), file2->GetFileSize() );
+		// Sort by filesize.
+		case  ID_SHARED_COL_SIZE:
+			return mod * CmpAny( file1->GetFileSize(), file2->GetFileSize() );
 
-		// Sort by filetype. Ascending.
-		case  ID_SHARED_COL_TYPE: return mod * GetFiletypeByName(file1->GetFileName()).CmpNoCase(GetFiletypeByName( file2->GetFileName()) );
+		// Sort by filetype.
+		case  ID_SHARED_COL_TYPE:
+			return mod * GetFiletypeByName(file1->GetFileName()).CmpNoCase(GetFiletypeByName( file2->GetFileName()) );
 
-		// Sort by priority. Ascending.
+		// Sort by priority.
 		case  ID_SHARED_COL_PRIO: {
 			int8 prioA = file1->GetUpPriority();
 			int8 prioB = file2->GetUpPriority();
@@ -371,20 +367,37 @@ int CSharedFilesCtrl::SortProc( long item1, long item2, long sortData )
 			return mod * CmpAny( ( prioB != PR_VERYLOW ? prioB : -1 ), ( prioA != PR_VERYLOW ? prioA : -1 ) );
 		}
 
-		// Sort by fileID. Ascending.
-		case  ID_SHARED_COL_ID: return mod * file1->GetFileHash().Encode().Cmp( file2->GetFileHash().Encode() );
+		// Sort by fileID.
+		case  ID_SHARED_COL_ID:
+			return mod * file1->GetFileHash().Encode().Cmp( file2->GetFileHash().Encode() );
 
-		// Sort by Requests this session. Ascending.
-		case  ID_SHARED_COL_REQ: return mod * CmpAny( file1->statistic.GetRequests(), file2->statistic.GetRequests() );
+		// Sort by Requests this session.
+		case  ID_SHARED_COL_REQ:
+			if (altSorting) {
+				return mod * CmpAny( file1->statistic.GetAllTimeRequests(), file2->statistic.GetAllTimeRequests() );
+			} else {
+				return mod * CmpAny( file1->statistic.GetRequests(), file2->statistic.GetRequests() );
+			}
 
 		// Sort by accepted requests. Ascending.
-		case  ID_SHARED_COL_AREQ: return mod * CmpAny( file1->statistic.GetAccepts(), file2->statistic.GetAccepts() );
+		case  ID_SHARED_COL_AREQ:
+			if (altSorting) {
+				return mod * CmpAny( file1->statistic.GetAllTimeAccepts(), file2->statistic.GetAllTimeAccepts() );
+			} else {
+				return mod * CmpAny( file1->statistic.GetAccepts(), file2->statistic.GetAccepts() );
+			}
 
 		// Sort by transferred. Ascending.
-		case  ID_SHARED_COL_TRA: return mod * CmpAny( file1->statistic.GetTransfered(), file2->statistic.GetTransfered() );
+		case  ID_SHARED_COL_TRA:
+			if (altSorting) {
+				return mod * CmpAny( file1->statistic.GetAllTimeTransfered(), file2->statistic.GetAllTimeTransfered() );
+			} else {
+				return mod * CmpAny( file1->statistic.GetTransfered(), file2->statistic.GetTransfered() );
+			}
 
 		// Complete sources asc
-		case ID_SHARED_COL_CMPL: return mod * CmpAny( file1->m_nCompleteSourcesCount, file2->m_nCompleteSourcesCount );
+		case ID_SHARED_COL_CMPL:
+			return mod * CmpAny( file1->m_nCompleteSourcesCount, file2->m_nCompleteSourcesCount );
 
 		// Folders ascending
 		case ID_SHARED_COL_PATH: {
@@ -398,15 +411,6 @@ int CSharedFilesCtrl::SortProc( long item1, long item2, long sortData )
 			return mod * file1->GetFilePath().Cmp( file2->GetFilePath() );
 		}
 		
-		// Sort by requests (All). Ascending.
-		case (2000 + ID_SHARED_COL_REQ): return mod * CmpAny( file1->statistic.GetAllTimeRequests(), file2->statistic.GetAllTimeRequests() );
-
-		// Sort by accepted requests (All). Ascending.
-		case (2000 + ID_SHARED_COL_AREQ): return mod * CmpAny( file1->statistic.GetAllTimeAccepts(), file2->statistic.GetAllTimeAccepts() );
-
-		// Sort by transferred (All). Ascending.
-		case (2000 + ID_SHARED_COL_TRA): return mod * CmpAny( file1->statistic.GetAllTimeTransfered(), file2->statistic.GetAllTimeTransfered() );
-	
 		default:
 			return 0;
 	}
@@ -478,52 +482,81 @@ void CSharedFilesCtrl::OnDrawItem( int item, wxDC* dc, const wxRect& rect, const
 	dc->DrawRectangle(rectHL);
 	dc->SetPen(*wxTRANSPARENT_PEN);
 
-	wxRect columnRect = rect;
-	
+	// Offset based on the height of the fonts
+	const int textVOffset = ( rect.GetHeight() - dc->GetCharHeight() ) / 2;
+	// Empty space to each side of a column
 	const int SPARE_PIXELS_HORZ	= 4;
 
-	// Offset based on the height of the fonts
-	const int textVOffset = ( rect.GetHeight() - GetFont().GetPointSize() ) / 2;
-
-	columnRect.SetLeft( columnRect.GetLeft() + SPARE_PIXELS_HORZ );
-	columnRect.SetWidth( columnRect.GetWidth()-2*SPARE_PIXELS_HORZ);
+	// The leftmost position of the current column
+	int columnLeft = 0;
+	
 	for ( int i = 0; i < GetColumnCount(); ++i ) {
-		wxListItem columnItem;
-		GetColumn(i, columnItem);
-		int width = columnItem.GetWidth();
-		columnRect.SetWidth(width-2*SPARE_PIXELS_HORZ);
+		const int columnWidth = GetColumnWidth(i);
 
-		if ( width ) {
+		if (columnWidth) {
+			wxRect columnRect(
+				columnLeft + SPARE_PIXELS_HORZ, rect.y,
+				columnWidth - 2 * SPARE_PIXELS_HORZ, rect.height);
+			
 			wxDCClipper clipper(*dc, columnRect);
 					
 			switch ( i ) {
 				case ID_SHARED_COL_PART: {
 					if ( file->GetPartCount() ) {
-						--columnRect.height;
-						++columnRect.y;
-						DrawAvailabilityBar(file, dc, columnRect );
-						++columnRect.height;
-						--columnRect.y;
+						wxRect barRect(columnRect.x, columnRect. y + 1, 
+							columnRect.width, columnRect.height - 2);
+						
+						DrawAvailabilityBar(file, dc, barRect);
 					}
 					break;
 				}
+				
+				case ID_SHARED_COL_NAME:
+					if (file->GetFileRating() || file->GetFileComment().Length()) {
+						int image = Client_CommentOnly_Smiley;
+						if (file->GetFileRating()) {
+							image = Client_InvalidRating_Smiley + file->GetFileRating() - 1;
+						}	
+							
+						wxASSERT(image >= Client_InvalidRating_Smiley);
+						wxASSERT(image <= Client_CommentOnly_Smiley);
+						
+						int imgWidth = 8;
+						if (file->GetFileRating() <= 1 || file->GetFileRating() == 5 ) {
+							imgWidth = 16;
+						} 
+						
+						theApp.amuledlg->imagelist.Draw(image, *dc, columnRect.x,
+								columnRect.y + 1, wxIMAGELIST_DRAW_TRANSPARENT);
+
+						// Move the text to the right
+						columnRect.x += (imgWidth + 4);
+					}
 
 				default: {
+					wxListItem columnItem;
 					columnItem.m_col = i;
 					columnItem.m_itemId = item;
 					GetItem(columnItem);
-					dc->DrawText(columnItem.m_text, columnRect.GetLeft(), columnRect.GetTop() + textVOffset );
+					dc->DrawText(columnItem.m_text, columnRect.x, columnRect.y + textVOffset );
 					break;
 				}
 			}
 		}
 
-		columnRect.SetLeft(columnRect.GetLeft()+width);
+		// Move to the next column
+		columnLeft += columnWidth;
 	}
 }
 
 
-bool CSharedFilesCtrl::AltSortAllowed( int column )
+wxString CSharedFilesCtrl::GetTTSText(unsigned item) const
+{
+	return GetItemText(item);
+}
+
+
+bool CSharedFilesCtrl::AltSortAllowed(unsigned column) const
 {
 	switch ( column ) {
 		case ID_SHARED_COL_REQ:
@@ -569,3 +602,36 @@ void CSharedFilesCtrl::OnGetRazorStats( wxCommandEvent& WXUNUSED(event) )
 		theApp.amuledlg->LaunchUrl(wxT("http://stats.razorback2.com/ed2khistory?ed2k=") + file->GetFileHash().Encode());
 	}
 }
+
+
+void CSharedFilesCtrl::OnRename( wxCommandEvent& WXUNUSED(event) )
+{
+	int item = GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
+	if ( item != -1 ) {
+		CKnownFile* file = (CKnownFile*)GetItemData(item);
+
+		// Currently renaming of completed files causes problem with kad
+		if (file->IsPartFile()) {
+			wxString newName = ::wxGetTextFromUser(
+				_("Enter new name for this file:"),
+				_("File rename"), file->GetFileName());
+				
+			if (!newName.IsEmpty() and (newName != file->GetFileName())) {
+				theApp.sharedfiles->RenameFile(file, newName);
+			}
+		}
+	}
+}
+
+
+void CSharedFilesCtrl::OnKeyPressed( wxKeyEvent& event )
+{
+	if (event.GetKeyCode() == WXK_F2) {
+		wxCommandEvent evt;
+		OnRename(evt);
+		
+		return;
+	}
+
+	event.Skip();
+}
