@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2004-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2004-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -19,7 +19,7 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #include <unistd.h>			// Needed for close(2) and sleep(3)
@@ -27,10 +27,6 @@
 #include <wx/gauge.h>
 #include <wx/textctrl.h>
 
-#ifdef __WXGTK__
-	#include <X11/Xlib.h>		// Needed for XParseGeometry
-#endif
-
 #ifdef HAVE_CONFIG_H
 	#include "config.h"			//   VERSION
 #endif
@@ -38,7 +34,6 @@
 #include <wx/filefn.h>
 #include <wx/ffile.h>
 #include <wx/file.h>
-#include <wx/filename.h>		// Needed for wxFileName::GetPathSeparator()
 #include <wx/log.h>
 #include <wx/timer.h>
 #include <wx/config.h>
@@ -54,11 +49,11 @@
 #include <wx/tokenzr.h>			// Needed for wxStringTokenizer
 #include <wx/msgdlg.h>			// Needed for wxMessageBox
 #include <wx/url.h>
+#include <wx/dataobj.h> 		// Needed on wxMotif
 
 #include "amule.h"			// Interface declarations.
 #include "GetTickCount.h"		// Needed for GetTickCount
 #include "Server.h"			// Needed for GetListName
-#include "CFile.h"			// Needed for CFile
 #include "OtherFunctions.h"		// Needed for GetTickCount
 #include "TransferWnd.h"		// Needed for CTransferWnd
 #include "SharedFilesWnd.h"		// Needed for CSharedFilesWnd
@@ -69,7 +64,7 @@
 #include "ClientCredits.h"		// Needed for CClientCreditsList
 #include "ClientUDPSocket.h"		// Needed for CClientUDPSocket
 #include "SharedFileList.h"		// Needed for CSharedFileList
-#include "ServerConnect.h"			// Needed for CServerConnect
+#include "ServerConnect.h"		// Needed for CServerConnect
 #include "ServerList.h"			// Needed for CServerList
 #include "KnownFileList.h"		// Needed for CKnownFileList
 #include "SearchList.h"			// Needed for CSearchList
@@ -78,7 +73,7 @@
 #include "ListenSocket.h"		// Needed for CListenSocket
 #include "ExternalConn.h"		// Needed for ExternalConn & MuleConnection
 #include "ServerSocket.h"		// Needed for CServerSocket
-#include "ServerUDPSocket.h"			// Needed for CServerUDPSocket
+#include "ServerUDPSocket.h"		// Needed for CServerUDPSocket
 #include "PartFile.h"			// Needed for CPartFile
 #include "AddFileThread.h"		// Needed for CAddFileThread
 #include "updownclient.h"		// Needed for CUpDownClient
@@ -93,6 +88,8 @@
 #include "DownloadListCtrl.h"		// Needed for CDownloadListCtrl
 #include "ClientListCtrl.h"
 #include "ChatWnd.h"
+#include <common/Format.h>
+#include "PartFileConvert.h"
 
 #ifdef __WXMAC__
 	#include <CoreFoundation/CFBundle.h>
@@ -100,6 +97,7 @@
 #endif
 
 #ifndef CLIENT_GUI
+#include "InternalEvents.h"		// Needed for wxEVT_*
 
 BEGIN_EVENT_TABLE(CamuleGuiApp, wxApp)
 
@@ -108,17 +106,17 @@ BEGIN_EVENT_TABLE(CamuleGuiApp, wxApp)
 	EVT_SOCKET(LISTENSOCKET_HANDLER, CamuleGuiApp::ListenSocketHandler)
 
 	// UDP Socket (servers)
-	EVT_SOCKET(SERVERUDPSOCKET_HANDLER, CamuleGuiApp::ServerUDPSocketHandler)
+	EVT_SOCKET(SERVERUDPSOCKET_HANDLER, CamuleGuiApp::UDPSocketHandler)
 	// UDP Socket (clients)
-	EVT_SOCKET(CLIENTUDPSOCKET_HANDLER, CamuleGuiApp::ClientUDPSocketHandler)
+	EVT_SOCKET(CLIENTUDPSOCKET_HANDLER, CamuleGuiApp::UDPSocketHandler)
 
 	// Socket timers (TCP + UDP)
-	EVT_TIMER(TM_TCPSOCKET, CamuleGuiApp::OnTCPTimer)
+	EVT_CUSTOM(wxEVT_AMULE_TIMER, TM_TCPSOCKET, CamuleGuiApp::OnTCPTimer)
 
 	// Core timer
-	EVT_TIMER(ID_CORETIMER, CamuleGuiApp::OnCoreTimer)
+	EVT_CUSTOM(wxEVT_AMULE_TIMER, ID_CORETIMER, CamuleGuiApp::OnCoreTimer)
 
-	EVT_CUSTOM(wxEVT_NOTIFY_EVENT, -1, CamuleGuiApp::OnNotifyEvent)
+	EVT_CUSTOM(wxEVT_MULE_NOTIFY_EVENT, -1, CamuleGuiApp::OnNotifyEvent)
 
 	// Async dns handling
 	EVT_CUSTOM(wxEVT_CORE_UDP_DNS_DONE, -1, CamuleGuiApp::OnUDPDnsDone)
@@ -148,29 +146,17 @@ IMPLEMENT_APP(CamuleGuiApp)
 
 #endif // CLIENT_GUI
 
-#ifndef __WXMSW__
-// Initialization of the static MyTimer member variables.
-uint32 MyTimer::tic32 = 0;
-uint64 MyTimer::tic64 = 0;
-
-
-// Global timer. Used to cache GetTickCount() results for better performance.
-class MyTimer* mytimer = NULL;
-#endif
-
 CamuleGuiBase::CamuleGuiBase()
 {
-#ifndef __WXMSW__
-	// Madcat - Initialize timer as the VERY FIRST thing to avoid any issues later.
-	// Kry - I love to init the vars on init, even before timer.
-	mytimer = new MyTimer();
-#endif
+
 }
 
 
 CamuleGuiBase::~CamuleGuiBase()
 {
-
+	#ifndef CLIENT_GUI
+	CPartFileConvert::StopThread();
+	#endif
 }
 
 
@@ -190,19 +176,15 @@ int CamuleGuiBase::InitGui(bool geometry_enabled, wxString &geom_string)
 
 	if ( geometry_enabled ) {
 		// I plan on moving this to a seperate function, as it just clutters up OnInit()
-#ifdef __WXGTK__
-		// Nothing we can do against this unicode2char either.
-		XParseGeometry(unicode2char(geom_string), &geometry_x, &geometry_y, &geometry_width, &geometry_height);
-		geometry_enabled = true;
-#elif defined (__WXMSW__)
 		/*
 		This implementation might work with mac, provided that the
 		SetSize() function works as expected.
 		*/
 
 		// Remove possible prefix
-		if ( geom_string.GetChar(0) == '=' )
+		if ( geom_string.GetChar(0) == '=' ) {
 			geom_string.Remove( 0, 1 );
+		}
 
 		// Stupid ToLong functions forces me to use longs =(
 		long width = geometry_width;
@@ -249,16 +231,21 @@ int CamuleGuiBase::InitGui(bool geometry_enabled, wxString &geom_string)
 				}
 			}
 		}
-#else
-		#warning Need to parse the geometry for non-GTK/WIN platforms
-#endif
 	}
 	
 	// Should default/last-used position be overridden?
-#ifdef CLIENT_GUI
-	m_FrameTitle = wxString::Format(wxT("aMule remote control %s"), wxT(VERSION));
+#ifdef CVSDATE
+	#ifdef CLIENT_GUI
+		m_FrameTitle = wxString::Format(wxT("aMule remote control %s %s"), wxT(VERSION), wxT(CVSDATE));
+	#else
+		m_FrameTitle = wxString::Format(wxT("aMule %s %s"), wxT(VERSION), wxT(CVSDATE));
+	#endif
 #else
-	m_FrameTitle = wxString::Format(wxT("aMule %s"), wxT(VERSION));
+	#ifdef CLIENT_GUI
+		m_FrameTitle = wxString::Format(wxT("aMule remote control %s"), wxT(VERSION));
+	#else
+		m_FrameTitle = wxString::Format(wxT("aMule %s"), wxT(VERSION));
+	#endif
 #endif
 	if ( geometry_enabled ) {
 		amuledlg = new CamuleDlg(NULL, m_FrameTitle,
@@ -310,23 +297,17 @@ void CamuleGuiApp::ShowAlert(wxString msg, wxString title, int flags)
 
 int CamuleGuiApp::OnExit()
 {
-	if (core_timer) {
-		// Stop the Core Timer
-		delete core_timer;
-	}
-	return CamuleApp::OnExit();
+	delete core_timer;
+	
+	return CamuleApp::OnExit();	
 }
 
 
-void CamuleGuiApp::ShutDown()
-{
+void CamuleGuiApp::ShutDown(wxCloseEvent &WXUNUSED(evt))
+{		
+	amuledlg->DlgShutDown();
 	amuledlg->Destroy();
 	CamuleApp::ShutDown();
-
-#ifndef __WXMSW__
-	delete mytimer;
-	mytimer = NULL;
-#endif
 }
 
 
@@ -339,13 +320,13 @@ bool CamuleGuiApp::OnInit()
 	}
 
 	// Create the Core timer
-	core_timer=new wxTimer(this,ID_CORETIMER);
+	core_timer = new CTimer(this,ID_CORETIMER);
 	if (!core_timer) {
 		printf("Fatal Error: Failed to create Core Timer");
 		OnExit();
 	}
 
-	// Start the Core Timer
+	// Start the Core and Gui timers
 
 	// Note: wxTimer can be off by more than 10% !!!
 	// In addition to the systematic error introduced by wxTimer, we are losing
@@ -356,17 +337,6 @@ bool CamuleGuiApp::OnInit()
 	// When adding functionality, assume that the timer is only approximately correct;
 	// for measurements, always use the system clock [::GetTickCount()].
 	core_timer->Start(100);
-
-	// Start the Gui Timer
-
-	// Note: wxTimer can be off by more than 10% !!!
-	// In addition to the systematic error introduced by wxTimer, we are losing
-	// timer cycles due to high CPU load.  I've observed about 0.5% random loss of cycles under
-	// low load, and more than 6% lost cycles with heavy download traffic and/or other tasks
-	// in the system, such as a video player or a VMware virtual machine.
-	// The upload queue process loop has now been rewritten to compensate for timer errors.
-	// When adding functionality, assume that the timer is only approximately correct;
-	// for measurements, always use the system clock [::GetTickCount()].
 	amuledlg->StartGuiTimer();
 
 #ifdef __WXMAC__
@@ -663,19 +633,23 @@ void CamuleGuiApp::NotifyEvent(const GUIEvent& event)
 				amuledlg->serverwnd->serverlistctrl->Thaw();
 			}
 			break;
-		case SERVER_UPDATEMYINFO:
+		case SERVER_UPDATEED2KINFO:
 			if ( amuledlg->serverwnd ) {
-				amuledlg->serverwnd->UpdateMyInfo();
+				amuledlg->serverwnd->UpdateED2KInfo();
+			}
+			break;
+		case SERVER_UPDATEKADINFO:
+			if ( amuledlg->serverwnd ) {
+				amuledlg->serverwnd->UpdateKadInfo();
 			}
 			break;
-
 		
 		// notification
-		case SHOW_NOTIFIER:
-			amuledlg->ShowNotifier(event.string_value,event.long_value,event.byte_value);
-			break;
 		case SHOW_CONN_STATE:
-			amuledlg->ShowConnectionState(event.byte_value, event.string_value);
+			#ifdef CLIENT_GUI
+				theApp.m_ConnState = event.long_value;
+			#endif
+			amuledlg->ShowConnectionState();
 			break;
 		case SHOW_QUEUE_COUNT:
 			if ( amuledlg->transferwnd ) {
@@ -688,9 +662,7 @@ void CamuleGuiApp::NotifyEvent(const GUIEvent& event)
 			}
 			break;
 		case SHOW_USER_COUNT:
-			amuledlg->ShowUserCount(((CServer*)event.ptr_value)->GetUsers(),
-									((CServer*)event.ptr_value)->GetFiles());
-			amuledlg->serverwnd->serverlistctrl->RefreshServer((CServer*)event.ptr_value);
+			amuledlg->ShowUserCount(event.string_value);
 			break;
 		case SHOW_GUI:
 			amuledlg->Show_aMule(true);
@@ -731,7 +703,7 @@ void CamuleGuiApp::NotifyEvent(const GUIEvent& event)
 		// chat window
 		case CHAT_REFRESH_FRIEND:
 			if ( amuledlg->chatwnd ) {
-				amuledlg->chatwnd->RefreshFriend(NULL, event.string_value, event.long_value, event.short_value);
+				amuledlg->chatwnd->RefreshFriend(CMD4Hash(), event.string_value, event.long_value, event.short_value);
 			}
 			break;
 		case CHAT_CONN_RESULT:
@@ -744,7 +716,32 @@ void CamuleGuiApp::NotifyEvent(const GUIEvent& event)
 				amuledlg->chatwnd->ProcessMessage(event.longlong_value, event.string_value);
 			}
 			break;
-			
+		case CATEGORY_ADD:
+			if ( amuledlg->transferwnd ) {
+				uint32 cat = event.long_value;
+				
+		        amuledlg->transferwnd->AddCategory(glob_prefs->GetCategory(cat));
+			}
+			break;
+		case CATEGORY_UPDATE:
+			if ( amuledlg->transferwnd ) {
+				uint32 cat = event.long_value;
+				
+				amuledlg->transferwnd->UpdateCategory(cat);
+				amuledlg->transferwnd->downloadlistctrl->Refresh();
+				amuledlg->searchwnd->UpdateCatChoice();
+			}
+			break;
+		case CATEGORY_DELETE:
+			if ( amuledlg->transferwnd ) {
+				uint32 cat = event.long_value;
+				
+				amuledlg->transferwnd->RemoveCategory(cat);
+				
+				amuledlg->searchwnd->UpdateCatChoice();
+			}
+			break;
+
 		// logging
 		case ADDDEBUGLOGLINE:
 		case ADDLOGLINE:
