@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2004-2005 Angel Vidal (Kry) ( kry@amule.org )
-// Copyright (c) 2004-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2004-2006 Angel Vidal (Kry) ( kry@amule.org )
+// Copyright (c) 2004-2006 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2003 Barry Dunne (http://www.emule-project.net)
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -21,7 +21,7 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 // Note To Mods //
@@ -44,26 +44,30 @@ namespace Kademlia {
 ////////////////////////////////////////
 
 #define SEARCHTOLERANCE				16777216
-#define K							10
+const unsigned int K	=						10;
 #define KBASE						4
 #define KK							5
 #define ALPHA_QUERY					3
 #define LOG_BASE_EXPONENT			5
 #define HELLO_TIMEOUT				20
-#define SEARCH_JUMPSTART			10
-#define SEARCH_LIFETIME				100
-#define SEARCHFILE_LIFETIME			100
-#define SEARCHKEYWORD_LIFETIME		100
-#define SEARCHNODE_LIFETIME			100
+#define SEARCH_JUMPSTART			2
+#define SEARCH_LIFETIME				45
+#define SEARCHFILE_LIFETIME			45
+#define SEARCHKEYWORD_LIFETIME		45
+#define SEARCHNOTES_LIFETIME		45
+#define SEARCHNODE_LIFETIME			45
 #define SEARCHNODECOMP_LIFETIME		10
 #define SEARCHSTOREFILE_LIFETIME	140
 #define SEARCHSTOREKEYWORD_LIFETIME	140
+#define SEARCHSTORENOTES_LIFETIME	100
 #define SEARCHFINDBUDDY_LIFETIME	100
 #define SEARCHFINDSOURCE_LIFETIME	45
 #define SEARCHFILE_TOTAL			300
 #define SEARCHKEYWORD_TOTAL			300
+#define SEARCHNOTES_TOTAL			50
 #define SEARCHSTOREFILE_TOTAL		10
 #define SEARCHSTOREKEYWORD_TOTAL	10
+#define SEARCHSTORENOTES_TOTAL		10
 #define SEARCHNODECOMP_TOTAL		10
 #define SEARCHFINDBUDDY_TOTAL		10
 #define SEARCHFINDSOURCE_TOTAL		20
