@@ -20,16 +20,12 @@
 /// You should have received a copy of the GNU General Public License
 /// along with this program; if not, write to the
 /// Free Software Foundation, Inc.,
-/// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+/// 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 #ifndef _ALCC_H
 #define _ALCC_H
 
-#ifdef __GNUG__
-#pragma interface "alcc.h"
-#endif
-
 // For compilers that support precompilation, includes "wx/wx.h"
 #include "wx/wxprec.h"
 
