@@ -22,7 +22,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the
  *  Free Software Foundation, Inc.,
- *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 
@@ -39,9 +39,9 @@
 #include "version.h"
 
 
-int create_html(char stats[20][80], char *lines[6], char template[120])
+int create_html(char *stats[20], char *lines[6], char template[120])
 {
-	// Strings
+	/* Strings */
 	char version[25], upload[25], download[25];
 	char *search[] = {"#VERSION#", "#CLIENT#", "#NICK#", "#UPLOADRATE#" ,
 		"#DOWNLOADRATE#" , "#QUEUE#" , "#NUMSHARE#" , "#SESSIONUP#" ,
@@ -49,14 +49,14 @@ int create_html(char stats[20][80], char *lines[6], char template[120])
 		"#PORT#" };
 
 	snprintf(version, 25, "cas %s", CAS_VERSION);
-	snprintf(upload, 25, "%s kB/s", stats[6]);
-	snprintf(download, 25, "%s kB/s", stats[5]);
+	snprintf(upload, 25, "%s kB/s", stats[7]);
+	snprintf(download, 25, "%s kB/s", stats[6]);
 
-	char *repl[] = { version , lines[0] , stats[9] , upload , download ,
-		stats[7] , stats[8] , stats[14] , stats[13] , stats[11] , stats[10] ,
+	char *repl[] = { version , lines[0] , stats[10] , upload , download ,
+		stats[8] , stats[9] , stats[15] , stats[14] , stats[12] , stats[11] ,
 		stats[1] , stats[2] , stats[3] };
 
-	// get some memory to read the template into
+	/* get some memory to read the template into */
 	int fdTmpl;
 	if ((fdTmpl = open(template, O_RDONLY)) < 0)
 	{
@@ -73,40 +73,40 @@ int create_html(char stats[20][80], char *lines[6], char template[120])
 	}
 	close(fdTmpl);
 
-	// 2 times the size of the template should be enougth
+	/* 2 times the size of the template should be enougth */
 	int size = sb.st_size*2;
-	char *mem = malloc(size);
+	char *mem = calloc(size, 1);
 	if (NULL == mem)
 	{
-		printf("coulnt malloc\n");
+		perror("Could not calloc\n");
 		exit(44);
 	}
-	memset(mem, '\0', size);
 
-	// read the template into the memory
+	/* read the template into the memory */
+	size_t len = 0;
 	int ler;
 	FILE *fTmpl = fopen(template,"r");
-	while ((ler=fgetc(fTmpl)) != EOF)
+	while ((ler=fgetc(fTmpl)) != EOF && len+1 < size)
 	{
-		sprintf(mem,"%s%c",mem,ler);
+		mem[len++] = ler;
 	}
 	fclose(fTmpl);
 	
-	//printf ("HTML: %s\n", mem);
+	/* printf ("HTML: %s\n", mem); */
 
 	int t;
 	for (t=0; t<=13; t++)
 	{
-		// replace the special tags
+		/* replace the special tags */
 		replace(mem, search[t], repl[t]);
 	}
 
-	//printf("FINAL: %s\n",mem);
+	/* printf("FINAL: %s\n",mem); */
 
-	char *path = get_path(".aMule/aMule-online-sign.html");
+	char *path = get_path("aMule-online-sign.html");
 	if (NULL == path)
 	{
-		printf("could not get the HTML path\n");
+		perror("could not get the HTML path\n");
 		free(mem);
 		return 0;
 	}
@@ -114,7 +114,7 @@ int create_html(char stats[20][80], char *lines[6], char template[120])
 	FILE *fHTML = NULL;
 	if ((fHTML = fopen(path, "w")) == NULL)
 	{
-		printf("Unable to create file\n");
+		perror("Unable to create file\n");
 		free(path);
 		free(mem);
 		exit(44);
